@@ -10,6 +10,49 @@ from pycloudlib.cloud import ImageType  # type: ignore
 DEFAULT_CONFIG_PATH = "~/.config/pycloudlib.toml"
 
 
+def cloud_factory(pro_config, cloud_name):
+    if cloud_name == "aws":
+        return EC2(
+            cloud_credentials_path=pro_config.cloud_credentials_path,
+            tag=pro_config.timed_job_tag,
+            timestamp_suffix=False,
+        )
+    if cloud_name == "azure":
+        return Azure(
+            cloud_credentials_path=pro_config.cloud_credentials_path,
+            tag=pro_config.timed_job_tag,
+            timestamp_suffix=False,
+        )
+    if cloud_name == "gcp":
+        return GCP(
+            cloud_credentials_path=pro_config.cloud_credentials_path,
+            tag=pro_config.timed_job_tag,
+            timestamp_suffix=False,
+        )
+    if cloud_name == "lxd-vm":
+        return LXDVirtualMachine(
+            cloud_credentials_path=pro_config.cloud_credentials_path,
+        )
+    if cloud_name == "lxd-container":
+        return LXDContainer(
+            cloud_credentials_path=pro_config.cloud_credentials_path,
+        )
+    raise RuntimeError("Invalid cloud name")
+
+
+class CloudManager:
+    def __init__(self, pro_config):
+        self.pro_config = pro_config
+        self.clouds = {}
+
+    def get(self, cloud_name):
+        cloud = self.clouds.get(cloud_name)
+        if cloud is None:
+            cloud = cloud_factory(self.pro_config, cloud_name)
+            self.clouds[cloud_name] = cloud
+        return cloud
+
+
 class Cloud:
     """Base class for cloud providers that should be tested through behave.
 
@@ -153,6 +196,7 @@ class Cloud:
             ephemeral=ephemeral,
             inbound_ports=inbound_ports,
         )
+        inst.wait()
         logging.info(
             "--- {} instance launched: {}.".format(self.name, inst.name)
         )
@@ -173,7 +217,11 @@ class Cloud:
         return instance.id
 
     def locate_image_name(
-        self, series: str, machine_type: str, daily: bool = True
+        self,
+        series: str,
+        machine_type: str,
+        daily: bool = True,
+        include_deprecated: bool = False,
     ) -> str:
         """Locate and return the image name to use for vm provision.
 
@@ -199,11 +247,17 @@ class Cloud:
 
         if daily:
             logging.debug("looking up daily image for {}".format(series))
-            return self.api.daily_image(release=series, image_type=image_type)
+            return self.api.daily_image(
+                release=series,
+                image_type=image_type,
+                include_deprecated=include_deprecated,
+            )
         else:
             logging.debug("looking up released image for {}".format(series))
             return self.api.released_image(
-                release=series, image_type=image_type
+                release=series,
+                image_type=image_type,
+                include_deprecated=include_deprecated,
             )
 
     def manage_ssh_key(
@@ -334,8 +388,19 @@ class EC2(Cloud):
                 daily = False
             else:
                 daily = True
+
+            include_deprecated = False
+            if series == "xenial":
+                logging.debug(
+                    "including deprecated images when locating xenial on aws"
+                )
+                include_deprecated = True
+
             image_name = self.locate_image_name(
-                series, machine_type, daily=daily
+                series,
+                machine_type,
+                daily=daily,
+                include_deprecated=include_deprecated,
             )
 
         logging.info(
@@ -650,7 +715,11 @@ class _LXD(Cloud):
         return instance.name
 
     def locate_image_name(
-        self, series: str, machine_type: str, daily: bool = True
+        self,
+        series: str,
+        machine_type: str,
+        daily: bool = True,
+        include_deprecated: bool = False,
     ) -> str:
         """Locate and return the image name to use for vm provision.
 
