@@ -1,5 +1,4 @@
 import datetime
-import itertools
 import logging
 import os
 import random
@@ -15,9 +14,10 @@ from behave.runner import Context
 
 import features.cloud as cloud
 from features.util import (
+    BUILDER_NAME_PREFIX,
     SUT,
     InstallationSource,
-    lxc_get_property,
+    landscape_reject_all_pending_computers,
     process_template_vars,
 )
 
@@ -43,22 +43,16 @@ class UAClientBehaveConfig:
         A valid contract token to use during attach scenarios
     :param contract_token_staging:
         A valid staging contract token to use during attach scenarios
-    :param image_clean:
-        This indicates whether the image created for this test run should be
-        cleaned up when all tests are complete.
-    :param machine_type:
-        The default machine_type to test: lxd-container, lxd-vm, azure.pro,
-            azure.generic, aws.pro or aws.generic
+    :param machine_types:
+        A comma-separated string of machine_types to test: lxd-container,
+            lxd-vm, azure.pro, azure.pro-fips, azure.generic, aws.pro,
+            aws.pro-fips, aws.generic, gcp.pro, gcp.pro-fips, gcp.generic
     :param private_key_file:
         Optional path to pre-existing private key file to use when connecting
         launched VMs via ssh.
     :param private_key_name:
         Optional name of the cloud's named private key object to use when
         connecting to launched VMs via ssh. Default: uaclient-integration.
-    :param reuse_image:
-        A string with an image name that should be used instead of building a
-        fresh image for this test run.   If specified, this image will not be
-        deleted.
     :param destroy_instances:
         This boolean indicates that test containers should be destroyed after
         the completion. Set to False to leave instances running.
@@ -75,7 +69,6 @@ class UAClientBehaveConfig:
     # environment variable input to the appropriate Python types for use within
     # the test framework
     boolean_options = [
-        "image_clean",
         "destroy_instances",
         "ephemeral_instance",
         "snapshot_strategy",
@@ -89,10 +82,10 @@ class UAClientBehaveConfig:
         "landscape_registration_key",
         "landscape_api_access_key",
         "landscape_api_secret_key",
-        "machine_type",
+        "machine_types",
+        "releases",
         "private_key_file",
         "private_key_name",
-        "reuse_image",
         "artifact_dir",
         "install_from",
         "custom_ppa",
@@ -118,15 +111,14 @@ class UAClientBehaveConfig:
         self,
         *,
         cloud_credentials_path: Optional[str] = None,
-        image_clean: bool = True,
         destroy_instances: bool = True,
         ephemeral_instance: bool = False,
         snapshot_strategy: bool = False,
         sbuild_output_to_terminal: bool = False,
-        machine_type: str = "lxd-container",
+        machine_types: Optional[str] = None,
+        releases: Optional[str] = None,
         private_key_file: Optional[str] = None,
         private_key_name: str = "uaclient-integration",
-        reuse_image: Optional[str] = None,
         contract_token: Optional[str] = None,
         contract_token_staging: Optional[str] = None,
         contract_token_staging_expired: Optional[str] = None,
@@ -140,7 +132,6 @@ class UAClientBehaveConfig:
         userdata_file: Optional[str] = None,
         check_version: Optional[str] = None,
         sbuild_chroot: Optional[str] = None,
-        cmdline_tags: List = []
     ) -> None:
         # First, store the values we've detected
         self.cloud_credentials_path = cloud_credentials_path
@@ -153,13 +144,9 @@ class UAClientBehaveConfig:
         self.landscape_registration_key = landscape_registration_key
         self.landscape_api_access_key = landscape_api_access_key
         self.landscape_api_secret_key = landscape_api_secret_key
-        self.image_clean = image_clean
         self.destroy_instances = destroy_instances
-        self.machine_type = machine_type
         self.private_key_file = private_key_file
         self.private_key_name = private_key_name
-        self.reuse_image = reuse_image
-        self.cmdline_tags = cmdline_tags
         self.artifact_dir = artifact_dir
         self.install_from = install_from
         self.custom_ppa = custom_ppa
@@ -167,13 +154,10 @@ class UAClientBehaveConfig:
         self.userdata_file = userdata_file
         self.check_version = check_version
         self.sbuild_chroot = sbuild_chroot
-        self.filter_series = set(
-            [
-                tag.split(".")[1]
-                for tag in cmdline_tags
-                if tag.startswith("series.") and "series.all" not in tag
-            ]
+        self.machine_types = (
+            machine_types.split(",") if machine_types else None
         )
+        self.releases = releases.split(",") if releases else None
         # Next, perform any required validation
         if install_from == InstallationSource.CUSTOM and not custom_ppa:
             logging.error(
@@ -188,12 +172,12 @@ class UAClientBehaveConfig:
             )
             sys.exit(1)
 
-        if self.reuse_image is not None:
-            if self.image_clean:
-                print(" Reuse_image specified, it will not be deleted.")
-
         ignore_vars = ()  # type: Tuple[str, ...]
-        if "pro" in self.machine_type:
+        if (
+            self.machine_types
+            and len(self.machine_types) == 1
+            and "pro" in self.machine_types[0]
+        ):
             ignore_vars += (
                 "UACLIENT_BEHAVE_CONTRACT_TOKEN",
                 "UACLIENT_BEHAVE_CONTRACT_TOKEN_STAGING",
@@ -203,8 +187,8 @@ class UAClientBehaveConfig:
             attr_name = env_name.replace("UACLIENT_BEHAVE_", "").lower()
             if getattr(self, attr_name):
                 print(
-                    " --- Ignoring {} because machine_type is {}".format(
-                        env_name, self.machine_type
+                    " --- Ignoring {} because machine_types is {}".format(
+                        env_name, self.machine_types
                     )
                 )
                 setattr(self, attr_name, None)
@@ -227,40 +211,9 @@ class UAClientBehaveConfig:
             random.choices(string.ascii_lowercase + string.digits, k=8)
         )
         timed_job_tag += "-" + random_suffix
+        self.timed_job_tag = timed_job_tag
 
-        self.clouds = {
-            "aws": cloud.EC2(
-                cloud_credentials_path=self.cloud_credentials_path,
-                tag=timed_job_tag,
-                timestamp_suffix=False,
-            ),
-            "azure": cloud.Azure(
-                cloud_credentials_path=self.cloud_credentials_path,
-                tag=timed_job_tag,
-                timestamp_suffix=False,
-            ),
-            "gcp": cloud.GCP(
-                cloud_credentials_path=self.cloud_credentials_path,
-                tag=timed_job_tag,
-                timestamp_suffix=False,
-            ),
-            "lxd-vm": cloud.LXDVirtualMachine(
-                cloud_credentials_path=self.cloud_credentials_path,
-            ),
-            "lxd-container": cloud.LXDContainer(
-                cloud_credentials_path=self.cloud_credentials_path,
-            ),
-        }
-        if "aws" in self.machine_type:
-            self.default_cloud = self.clouds["aws"]
-        elif "azure" in self.machine_type:
-            self.default_cloud = self.clouds["azure"]
-        elif "gcp" in self.machine_type:
-            self.default_cloud = self.clouds["gcp"]
-        elif "lxd-vm" in self.machine_type:
-            self.default_cloud = self.clouds["lxd-vm"]
-        else:
-            self.default_cloud = self.clouds["lxd-container"]
+        self.clouds = cloud.CloudManager(self)
 
         # Finally, print the config options.  This helps users debug the use of
         # config options, and means they'll be included in test logs in CI.
@@ -281,13 +234,7 @@ class UAClientBehaveConfig:
         kwargs = (
             {}
         )  # type: Dict[str, Union[str, bool, List, InstallationSource]]
-        # Preserve cmdline_tags for reference
-        if not config.tags.ands:
-            kwargs["cmdline_tags"] = []
-        else:
-            kwargs["cmdline_tags"] = list(
-                itertools.chain.from_iterable(config.tags.ands)
-            )
+
         for key, value in os.environ.items():
             if not key.startswith(cls.prefix):
                 continue
@@ -309,7 +256,13 @@ class UAClientBehaveConfig:
                 kwargs[key] = bool_value
 
         if "install_from" in kwargs:
-            kwargs["install_from"] = InstallationSource(kwargs["install_from"])
+            if str(kwargs["install_from"]).startswith("ppa:"):
+                kwargs["custom_ppa"] = kwargs["install_from"]
+                kwargs["install_from"] = InstallationSource.CUSTOM
+            else:
+                kwargs["install_from"] = InstallationSource(
+                    kwargs["install_from"]
+                )
 
         return cls(**kwargs)  # type: ignore
 
@@ -336,35 +289,16 @@ def before_all(context: Context) -> None:
         for key, value in userdata.items():
             logging.debug("   - {} = {}".format(key, value))
             print("   - {} = {}".format(key, value))
-    context.series_image_name = {}
-    context.series_reuse_image = ""
     context.pro_config = UAClientBehaveConfig.from_environ(context.config)
     context.snapshots = {}
     context.machines = {}
 
-    if context.pro_config.reuse_image:
-        series = lxc_get_property(
-            context.pro_config.reuse_image, property_name="series", image=True
-        )
-        machine_type = lxc_get_property(
-            context.pro_config.reuse_image,
-            property_name="machine_type",
-            image=True,
-        )
-        if machine_type:
-            print("Found machine_type: {vm_type}".format(vm_type=machine_type))
-        if series is not None:
-            context.series_reuse_image = series
-            context.series_image_name[series] = context.pro_config.reuse_image
-        else:
-            print(" Could not check image series. It will not be used. ")
-            context.pro_config.reuse_image = None
 
-
-def _should_skip_tags(context: Context, tags: List) -> str:
-    """Return a reason if a feature or scenario should be skipped"""
-    machine_type = getattr(context.pro_config, "machine_type", "")
-    machine_types = []
+def _should_skip_config_tags(context: Context, tags: List) -> str:
+    """
+    Return a reason if a feature or scenario should be skipped based on
+    missing but required config tags
+    """
 
     for tag in tags:
         parts = tag.split(".")
@@ -373,26 +307,14 @@ def _should_skip_tags(context: Context, tags: List) -> str:
         val = context.pro_config
         for idx, attr in enumerate(parts[2:], 1):
             val = getattr(val, attr, None)
-            if attr == "machine_type":
-                curr_machine_type = ".".join(parts[idx + 2 :])
-                machine_types.append(curr_machine_type)
-                if curr_machine_type == machine_type:
-                    return ""
-
-                break
             if val is None:
                 return "Skipped: tag value was None: {}".format(tag)
 
-    if machine_types:
-        return "Skipped: machine type {} was not found in tags:\n {}".format(
-            machine_type, ", ".join(machine_types)
-        )
-
     return ""
 
 
 def before_feature(context: Context, feature: Feature):
-    reason = _should_skip_tags(context, feature.tags)
+    reason = _should_skip_config_tags(context, feature.tags)
     if reason:
         feature.skip(reason=reason)
 
@@ -400,58 +322,67 @@ def before_feature(context: Context, feature: Feature):
 def before_scenario(context: Context, scenario: Scenario):
     context.stored_vars = {}
 
-    reason = _should_skip_tags(context, scenario.effective_tags)
+    reason = _should_skip_config_tags(context, scenario.effective_tags)
     if reason:
         scenario.skip(reason=reason)
         return
 
-    filter_series = context.pro_config.filter_series
-    given_a_series_match = re.match(
-        "a `([a-z]*)` machine with ubuntu-advantage-tools installed",
+    # Determine release and machine_type of this scenario.
+    # First check outline example table row.
+    # Then override with what is passed directly to the "Give a machine" step
+    # if applicable.
+    scenario_release = None
+    scenario_machine_type = None
+
+    if hasattr(scenario, "_row") and scenario._row is not None:
+        scenario_release = scenario._row.get("release")
+        scenario_machine_type = scenario._row.get("machine_type")
+
+    given_a_series_machine_type_match = re.match(
+        "a `(.*)` `(.*)` machine with ubuntu-advantage-tools installed",
         scenario.steps[0].name,
     )
-    if filter_series and given_a_series_match:
-        series = given_a_series_match.group(1)
-        if series and series not in filter_series:
-            scenario.skip(
-                reason=(
-                    "Skipping scenario outline series `{series}`."
-                    " Cmdline provided @series tags: {cmdline_series}".format(
-                        series=series, cmdline_series=filter_series
-                    )
+    if given_a_series_machine_type_match:
+        step_release = given_a_series_machine_type_match.group(1)
+        if step_release != "<release>":
+            scenario_release = step_release
+        step_machine_type = given_a_series_machine_type_match.group(2)
+        if step_machine_type != "<machine_type>":
+            scenario_machine_type = step_machine_type
+
+    releases = context.pro_config.releases
+    if releases and scenario_release not in releases:
+        scenario.skip(
+            reason=(
+                "Scenario release is `{}`, but releases filter set"
+                " to {} - skipping.".format(scenario_release, releases)
+            )
+        )
+        return
+    machine_types = context.pro_config.machine_types
+    if machine_types and scenario_machine_type not in machine_types:
+        scenario.skip(
+            reason=(
+                "Scenario machine_type is `{}`, but machine_types filter set"
+                " to {} - skipping.".format(
+                    scenario_machine_type, context.pro_config.machine_types
                 )
             )
+        )
+        return
+
+    install_from = context.pro_config.install_from
+    if install_from == InstallationSource.LOCAL:
+        if "skip_local_environment" in scenario.effective_tags:
+            scenario.skip(
+                reason="Scenario does not support install_from local"
+            )
             return
 
-    if hasattr(scenario, "_row") and scenario._row is not None:
-        row_release = scenario._row.get("release")
-        if (
-            row_release
-            and len(filter_series) > 0
-            and row_release not in filter_series
-        ):
+    if install_from == InstallationSource.PREBUILT:
+        if "skip_prebuilt_environment" in scenario.effective_tags:
             scenario.skip(
-                reason=(
-                    "Skipping scenario outline series `{series}`."
-                    " Cmdline provided @series tags: {cmdline_series}".format(
-                        series=row_release, cmdline_series=filter_series
-                    )
-                )
-            )
-            return
-        row_machine_type = scenario._row.get("machine_type")
-        if (
-            row_machine_type
-            and context.pro_config.machine_type != "any"
-            and row_machine_type != context.pro_config.machine_type
-        ):
-            scenario.skip(
-                reason=(
-                    "Skipping scenario outline machine_type `{}`."
-                    " Cmdline provided machine_type: {}".format(
-                        row_machine_type, context.pro_config.machine_type
-                    )
-                )
+                reason="Scenario does not support install_from prebuilt"
             )
             return
 
@@ -527,36 +458,40 @@ def after_step(context, step):
 
 
 def after_all(context):
-    if context.pro_config.image_clean:
-        for key, image in context.series_image_name.items():
-            if key == context.series_reuse_image:
-                logging.info(
-                    " Not deleting this image: ",
-                    context.series_image_name[key],
-                )
-            else:
-                context.pro_config.default_cloud.api.delete_image(image)
-
     if context.pro_config.destroy_instances:
         try:
-            if context.pro_config.default_cloud._ssh_key_managed:
-                key_pair = context.pro_config.default_cloud.api.key_pair
-                os.remove(key_pair.private_key_path)
-                os.remove(key_pair.public_key_path)
+            for cloud_instance in context.pro_config.clouds.clouds.values():
+                if cloud_instance._ssh_key_managed:
+                    key_pair = cloud_instance.api.key_pair
+                    os.remove(key_pair.private_key_path)
+                    os.remove(key_pair.public_key_path)
 
         except Exception as e:
             logging.error(
                 "Failed to delete instance ssh keys:\n{}".format(str(e))
             )
 
-    if "builder" in context.snapshots:
+    # Builder snapshots don't get an auto-cleanup function, so clean them here
+    builder_snapshots = [
+        name
+        for name in context.snapshots
+        if name.startswith(BUILDER_NAME_PREFIX)
+    ]
+    for snapshot in builder_snapshots:
+        cloud = context.snapshots[snapshot].cloud
         try:
-            context.pro_config.default_cloud.api.delete_image(
-                context.snapshots["builder"]
+            context.pro_config.clouds.get(cloud).api.delete_image(
+                context.snapshots[snapshot].name
             )
         except RuntimeError as e:
             logging.error(
                 "Failed to delete image: {}\n{}".format(
-                    context.snapshots["builder"], str(e)
+                    context.snapshots[snapshot].name, str(e)
                 )
             )
+
+    if (
+        context.pro_config.landscape_api_access_key
+        and context.pro_config.landscape_api_secret_key
+    ):
+        landscape_reject_all_pending_computers(context)
