@@ -1,24 +1,29 @@
 import json
+import logging
+import re
+import textwrap
 
 import jsonschema  # type: ignore
 import yaml
 from behave import then, when
-from hamcrest import (
-    assert_that,
-    contains_string,
-    equal_to,
-    matches_regexp,
-    not_,
-)
+from hamcrest import assert_that, contains_string, equal_to, not_
 
 from features.steps.shell import when_i_run_command
 from features.util import SafeLoaderWithoutDatetime, process_template_vars
 
 
-@then("I will see the following on stdout")
-def then_i_will_see_on_stdout(context):
+@then("I will see the following on {stream}")
+def then_i_will_see_on_stream(context, stream):
+    content = getattr(context.process, stream).strip()
     text = process_template_vars(context, context.text)
-    assert_that(context.process.stdout.strip(), equal_to(text))
+    logging.debug("repr(expected): %r", text)
+    logging.debug("repr(actual): %r", content)
+    if not text == content:
+        raise AssertionError(
+            "Expected to find exactly:\n{}\nBut got:\n{}".format(
+                textwrap.indent(text, "  "), textwrap.indent(content, "  ")
+            )
+        )
 
 
 @then("if `{value1}` in `{value2}` and stdout matches regexp")
@@ -48,21 +53,39 @@ def then_not_in_conditional_stdout_does_not_match_regexp(
 def then_stream_does_not_match_regexp(context, stream):
     text = process_template_vars(context, context.text)
     content = getattr(context.process, stream).strip()
-    assert_that(content, not_(matches_regexp(text)))
+    if re.compile(text).search(content) is not None:
+        raise AssertionError(
+            "Expected to NOT match regexp:\n{}\nBut got:\n{}".format(
+                textwrap.indent(text, "  "), textwrap.indent(content, "  ")
+            )
+        )
 
 
 @then("{stream} matches regexp")
 def then_stream_matches_regexp(context, stream):
     content = getattr(context.process, stream).strip()
     text = process_template_vars(context, context.text)
-    assert_that(content, matches_regexp(text))
+    if re.compile(text).search(content) is None:
+        raise AssertionError(
+            "Expected to match regexp:\n{}\nBut got:\n{}".format(
+                textwrap.indent(text, "  "), textwrap.indent(content, "  ")
+            )
+        )
 
 
 @then("{stream} contains substring")
 def then_stream_contains_substring(context, stream):
     content = getattr(context.process, stream).strip()
     text = process_template_vars(context, context.text)
-    assert_that(content, contains_string(text))
+    if text not in content:
+        raise AssertionError(
+            (
+                "Expected to find substring:\n{}\n"
+                + "But couldn't find it in:\n{}"
+            ).format(
+                textwrap.indent(text, "  "), textwrap.indent(content, "  ")
+            )
+        )
 
 
 @then("{stream} does not contain substring")
@@ -70,12 +93,15 @@ def then_stream_not_contains_substring(context, stream):
     content = getattr(context.process, stream).strip()
     text = process_template_vars(context, context.text)
     assert_that(content, not_(contains_string(text)))
-
-
-@then("I will see the following on stderr")
-def then_i_will_see_on_stderr(context):
-    text = process_template_vars(context, context.text)
-    assert_that(context.process.stderr.strip(), equal_to(text))
+    if text in content:
+        raise AssertionError(
+            (
+                "Expected to NOT find substring:\n{}\n"
+                + "But did find it in:\n{}"
+            ).format(
+                textwrap.indent(text, "  "), textwrap.indent(content, "  ")
+            )
+        )
 
 
 @then("I will see the uaclient version on stdout")
@@ -130,3 +156,12 @@ def root_vs_nonroot_cmd_comparison(context, cmd):
 
     assert_that(root_status_stdout, nonroot_status_stdout)
     assert root_status_stderr == nonroot_status_stderr
+
+
+@when("I verify that `{field}` field is redacted in the logs")
+def i_verify_field_is_redacted_in_the_logs(context, field):
+    when_i_run_command(
+        context, "cat /var/log/ubuntu-advantage.log", "with sudo"
+    )
+    context.text = field + "<REDACTED>"
+    then_stream_contains_substring(context, "stdout")
