@@ -3,15 +3,23 @@ import shlex
 import subprocess
 import time
 
-from behave import then, when
+from behave import step, then, when
 from hamcrest import assert_that, equal_to
 
 from features.util import SUT, process_template_vars
 
 
 @when("I run `{command}` {user_spec}, retrying exit [{exit_codes}]")
-def when_i_retry_run_command(context, command, user_spec, exit_codes):
-    when_i_run_command(context, command, user_spec, verify_return=False)
+def when_i_retry_run_command(
+    context, command, user_spec, exit_codes, machine_name=SUT
+):
+    when_i_run_command(
+        context,
+        command,
+        user_spec,
+        verify_return=False,
+        machine_name=machine_name,
+    )
     retries = [5, 5, 10]  # Sleep times to wait between retries
     while len(retries) > 0 and str(
         context.process.returncode
@@ -22,7 +30,13 @@ def when_i_retry_run_command(context, command, user_spec, exit_codes):
                 exit_code=context.process.returncode, command=command
             )
         )
-        when_i_run_command(context, command, user_spec, verify_return=False)
+        when_i_run_command(
+            context,
+            command,
+            user_spec,
+            verify_return=False,
+            machine_name=machine_name,
+        )
     logging.warning(
         "Exhausted retries waiting for exit codes: %s. Final exit code: %d",
         exit_codes,
@@ -31,6 +45,12 @@ def when_i_retry_run_command(context, command, user_spec, exit_codes):
     assert_that(context.process.returncode, equal_to(0))
 
 
+@when("I run `{command}` `{user_spec}` and the following stdin")
+def when_i_run_command_with_long_stdin(context, command, user_spec):
+    text = process_template_vars(context, context.text)
+    when_i_run_command(context, command, user_spec, stdin=text)
+
+
 @when("I run `{command}` {user_spec}")
 @when("I run `{command}` `{user_spec}` on the `{machine_name}` machine")
 @when("I run `{command}` `{user_spec}` and stdin `{stdin}`")
@@ -90,17 +110,19 @@ def then_i_should_see_that_the_command_is_not_found(context, cmd_name):
     assert_that(expected_return, equal_to(actual_return))
 
 
-@then("I verify that running `{cmd_name}` `{spec}` exits `{exit_codes}`")
-def then_i_verify_that_running_cmd_with_spec_exits_with_codes(
+@step(
+    "I verify that running `{cmd_name}` `{spec}` and the following stdin exits `{exit_codes}`"  # noqa
+)
+def i_verify_that_running_cmd_with_spec_and_stdin_exits_with_codes(
     context, cmd_name, spec, exit_codes
 ):
-    when_i_run_command(context, cmd_name, spec, verify_return=False)
-    logging.debug("got return code: %d", context.process.returncode)
-    expected_codes = exit_codes.split(",")
-    assert str(context.process.returncode) in expected_codes
+    text = process_template_vars(context, context.text)
+    then_i_verify_that_running_cmd_with_spec_and_stdin_exits_with_codes(
+        context, cmd_name, spec, text, exit_codes
+    )
 
 
-@when(
+@step(
     "I verify that running `{cmd_name}` `{spec}` and stdin `{stdin}` exits `{exit_codes}`"  # noqa
 )
 def then_i_verify_that_running_cmd_with_spec_and_stdin_exits_with_codes(
@@ -111,16 +133,27 @@ def then_i_verify_that_running_cmd_with_spec_and_stdin_exits_with_codes(
     )
 
     expected_codes = exit_codes.split(",")
-    assert str(context.process.returncode) in expected_codes
+    if str(context.process.returncode) not in expected_codes:
+        raise AssertionError(
+            "Expected exit code in: {} but got {}".format(
+                expected_codes, context.process.returncode
+            )
+        )
 
 
-@when("I verify that running `{cmd_name}` `{spec}` exits `{exit_codes}`")
-def when_i_verify_that_running_cmd_with_spec_exits_with_codes(
+@step("I verify that running `{cmd_name}` `{spec}` exits `{exit_codes}`")
+def then_i_verify_that_running_cmd_with_spec_exits_with_codes(
     context, cmd_name, spec, exit_codes
 ):
-    then_i_verify_that_running_cmd_with_spec_exits_with_codes(
-        context, cmd_name, spec, exit_codes
-    )
+    when_i_run_command(context, cmd_name, spec, verify_return=False)
+    logging.debug("got return code: %d", context.process.returncode)
+    expected_codes = exit_codes.split(",")
+    if str(context.process.returncode) not in expected_codes:
+        raise AssertionError(
+            "Expected exit code in: {} but got {}".format(
+                expected_codes, context.process.returncode
+            )
+        )
 
 
 def get_command_prefix_for_user_spec(user_spec):
