@@ -14,6 +14,7 @@ from uaclient.entitlements.entitlement_status import (
     CanEnableFailure,
     CanEnableFailureReason,
 )
+from uaclient.testing import fakes
 from uaclient.testing.helpers import does_not_raise
 
 M_PATH = "uaclient.api.u.pro.attach.auto.full_auto_attach.v1."
@@ -71,8 +72,10 @@ class TestEnableServicesByName:
                 False,
                 [
                     (True, None),
-                    exceptions.UserFacingError("msg"),
-                    exceptions.UserFacingError("msg", "code"),
+                    exceptions.EntitlementNotFoundError(
+                        entitlement_name="name"
+                    ),
+                    fakes.FakeUbuntuProError(),
                 ],
                 [
                     mock.call(
@@ -92,8 +95,13 @@ class TestEnableServicesByName:
                     ),
                 ],
                 [
-                    ("esm-infra", messages.NamedMessage("unknown", "msg")),
-                    ("livepatch", messages.NamedMessage("code", "msg")),
+                    (
+                        "esm-infra",
+                        messages.E_ENTITLEMENT_NOT_FOUND.format(
+                            entitlement_name="name"
+                        ),
+                    ),
+                    ("livepatch", fakes.FakeUbuntuProError._msg),
                 ],
             ),
             # fail via return
@@ -166,12 +174,12 @@ class TestFullAutoAttachV1:
     @mock.patch(
         "uaclient.actions.enable_entitlement_by_name",
     )
-    @mock.patch("uaclient.actions.get_cloud_instance")
+    @mock.patch("uaclient.clouds.identity.cloud_instance_factory")
     @mock.patch("uaclient.actions.auto_attach")
     def test_error_invalid_ent_names(
         self,
         _auto_attach,
-        _get_cloud_instance,
+        _cloud_instance_factory,
         m_enable_ent_by_name,
         _m_update_activity_token,
         _notice_remove,
@@ -203,12 +211,12 @@ class TestFullAutoAttachV1:
         "uaclient.actions.enable_entitlement_by_name",
         return_value=(False, None),
     )
-    @mock.patch("uaclient.actions.get_cloud_instance")
+    @mock.patch("uaclient.clouds.identity.cloud_instance_factory")
     @mock.patch("uaclient.actions.auto_attach")
     def test_error_full_auto_attach_fail(
         self,
         _auto_attach,
-        _get_cloud_instance,
+        _cloud_instance_factory,
         enable_ent_by_name,
         _m_update_activity_token,
         _notice_remove,
@@ -226,13 +234,15 @@ class TestFullAutoAttachV1:
         assert 4 == enable_ent_by_name.call_count
 
     @mock.patch(
-        "uaclient.lock.SpinLock.__enter__",
+        "uaclient.lock.RetryLock.__enter__",
         side_effect=[
-            exceptions.LockHeldError("request", "holder", 10),
+            exceptions.LockHeldError(
+                lock_request="request", lock_holder="holder", pid=10
+            ),
         ],
     )
     def test_lock_held(
-        self, _m_spinlock_enter, _notice_remove, _notice_read, FakeConfig
+        self, _m_retrylock_enter, _notice_remove, _notice_read, FakeConfig
     ):
         with pytest.raises(exceptions.LockHeldError):
             _full_auto_attach(FullAutoAttachOptions, FakeConfig())
@@ -265,7 +275,7 @@ class TestFullAutoAttachV1:
                 [],
                 pytest.raises(exceptions.AlreadyAttachedError),
                 False,
-                messages.ALREADY_ATTACHED.format(
+                messages.E_ALREADY_ATTACHED.format(
                     account_name="test_account"
                 ).msg,
                 None,
@@ -280,7 +290,7 @@ class TestFullAutoAttachV1:
                 [],
                 pytest.raises(exceptions.AutoAttachDisabledError),
                 False,
-                messages.AUTO_ATTACH_DISABLED_ERROR.msg,
+                messages.E_AUTO_ATTACH_DISABLED_ERROR.msg,
                 None,
             ),
             # success no options
@@ -354,7 +364,7 @@ class TestFullAutoAttachV1:
                 ],
                 pytest.raises(exceptions.EntitlementsNotEnabledError),
                 True,
-                messages.ENTITLEMENTS_NOT_ENABLED_ERROR.msg,
+                messages.E_ENTITLEMENTS_NOT_ENABLED_ERROR.msg,
                 None,
             ),
         ],
@@ -364,12 +374,12 @@ class TestFullAutoAttachV1:
     )
     @mock.patch(M_PATH + "_enable_services_by_name")
     @mock.patch(M_PATH + "actions.auto_attach")
-    @mock.patch(M_PATH + "actions.get_cloud_instance")
+    @mock.patch(M_PATH + "identity.cloud_instance_factory")
     @mock.patch(M_PATH + "util.is_config_value_true")
     def test_full_auto_attach_v1(
         self,
         m_is_config_value_true,
-        m_get_cloud_instance,
+        m_cloud_instance_factory,
         m_auto_attach,
         m_enable_services_by_name,
         m_update_activity_token,
