@@ -9,6 +9,7 @@ import pytest
 
 from uaclient import event_logger, http, messages, util
 from uaclient.cli import (
+    _post_cli_attach,
     action_attach,
     attach_parser,
     get_parser,
@@ -30,7 +31,7 @@ HELP_OUTPUT = textwrap.dedent(
     """\
 usage: pro attach <token> [flags]
 
-Attach this machine to Ubuntu Pro with a token obtained from:
+Attach this machine to an Ubuntu Pro subscription with a token obtained from:
 https://ubuntu.com/pro/dashboard
 
 When running this command without a token, it will generate a short code
@@ -176,10 +177,12 @@ class TestActionAttach:
         }
         assert expected == json.loads(capsys.readouterr()[0])
 
+    @mock.patch("time.sleep")
     @mock.patch("uaclient.system.subp")
     def test_lock_file_exists(
         self,
         m_subp,
+        m_sleep,
         capsys,
         FakeConfig,
         event,
@@ -189,7 +192,7 @@ class TestActionAttach:
         cfg.write_cache("lock", "123:pro disable")
         with pytest.raises(LockHeldError) as exc_info:
             action_attach(mock.MagicMock(), cfg=cfg)
-        assert [mock.call(["ps", "123"])] == m_subp.call_args_list
+        assert [mock.call(["ps", "123"])] * 12 == m_subp.call_args_list
         assert (
             "Unable to perform: pro attach.\n"
             "Operation in progress: pro disable (pid:123)"
@@ -231,6 +234,25 @@ class TestActionAttach:
         }
         assert expected == json.loads(capsys.readouterr()[0])
 
+    @mock.patch(
+        "uaclient.status.format_tabular", return_value="mock_tabular_status"
+    )
+    @mock.patch("uaclient.actions.status", return_value=("", 0))
+    @mock.patch(M_PATH + "daemon")
+    def test_post_cli_attach(
+        self, m_daemon, m_status, m_format_tabular, capsys, FakeConfig
+    ):
+        cfg = FakeConfig.for_attached_machine()
+        _post_cli_attach(cfg)
+
+        assert [mock.call()] == m_daemon.stop.call_args_list
+        assert [mock.call(cfg)] == m_daemon.cleanup.call_args_list
+        assert [mock.call(cfg)] == m_status.call_args_list
+        assert [mock.call("")] == m_format_tabular.call_args_list
+        out, _ = capsys.readouterr()
+        assert "This machine is now attached to 'test_contract'" in out
+        assert "mock_tabular_status" in out
+
     @mock.patch(
         M_PATH + "contract.UAContractClient.update_activity_token",
     )
@@ -239,12 +261,10 @@ class TestActionAttach:
     @mock.patch("uaclient.files.notices.NoticesManager.remove")
     @mock.patch("uaclient.timer.update_messaging.update_motd_messages")
     @mock.patch(M_PATH + "contract.UAContractClient.add_contract_machine")
-    @mock.patch("uaclient.actions.status", return_value=("", 0))
-    @mock.patch("uaclient.status.format_tabular")
+    @mock.patch(M_PATH + "_post_cli_attach")
     def test_happy_path_with_token_arg(
         self,
-        m_format_tabular,
-        m_status,
+        m_post_cli,
         contract_machine_attach,
         m_update_apt_and_motd_msgs,
         _m_remove_notice,
@@ -270,46 +290,13 @@ class TestActionAttach:
         ret = action_attach(args, cfg)
 
         assert 0 == ret
-        assert 1 == m_status.call_count
-        assert 1 == m_format_tabular.call_count
         expected_calls = [
             mock.call(contract_token=token, attachment_dt=mock.ANY)
         ]
         assert expected_calls == contract_machine_attach.call_args_list
         assert [mock.call(cfg)] == m_update_apt_and_motd_msgs.call_args_list
         assert 1 == m_update_activity_token.call_count
-
-        # We need to do that since all config objects in this
-        # test will share the same data dir. Since this will
-        # test a successful attach, in the end we write a machine token
-        # file, which will make all other cfg objects here to report
-        # as attached
-        cfg.delete_cache()
-        cfg.machine_token_file.delete()
-
-        cfg = FakeConfig()
-        args = mock.MagicMock(token=token, attach_config=None)
-        with mock.patch.object(
-            event, "_event_logger_mode", event_logger.EventLoggerMode.JSON
-        ):
-            with mock.patch.object(
-                cfg, "check_lock_info"
-            ) as m_check_lock_info:
-                m_check_lock_info.return_value = (0, "lock_holder")
-                fake_stdout = io.StringIO()
-                with contextlib.redirect_stdout(fake_stdout):
-                    main_error_handler(action_attach)(args, cfg)
-
-        expected = {
-            "_schema_version": event_logger.JSON_SCHEMA_VERSION,
-            "result": "success",
-            "errors": [],
-            "failed_services": [],
-            "needs_reboot": False,
-            "processed_services": [],
-            "warnings": [],
-        }
-        assert expected == json.loads(fake_stdout.getvalue())
+        assert [mock.call(cfg)] == m_post_cli.call_args_list
 
     @pytest.mark.parametrize("auto_enable", (True, False))
     @mock.patch(
