@@ -1,6 +1,8 @@
 import datetime
 import io
 import logging
+import os
+import shutil
 from enum import Enum
 from typing import Any, Dict
 
@@ -17,6 +19,11 @@ except ImportError:
     raise
 
 
+shutil.get_terminal_size = mock.MagicMock(
+    return_value=os.terminal_size((80, 20))
+)
+
+
 @pytest.yield_fixture(scope="session", autouse=True)
 def _subp():
     """
