@@ -26,7 +26,7 @@ def time_mock_side_effect_increment_by(increment):
 
 @mock.patch(M_PATH + "LOG.debug")
 @mock.patch(M_PATH + "actions.auto_attach")
-@mock.patch(M_PATH + "lock.SpinLock")
+@mock.patch(M_PATH + "lock.RetryLock")
 class TestAttemptAutoAttach:
     def test_success(
         self, m_spin_lock, m_auto_attach, m_log_debug, FakeConfig
@@ -142,7 +142,7 @@ class TestPollForProLicense:
                 False,
                 False,
                 True,
-                exceptions.CloudFactoryError("none"),
+                exceptions.CloudFactoryNoCloudError(),
                 None,
                 None,
                 None,
@@ -325,7 +325,8 @@ class TestPollForProLicense:
                 [
                     mock.call(
                         "wait_for_change returned quickly and no pro license"
-                        " present. Waiting 123 seconds before polling again"
+                        " present. Waiting %d seconds before polling again",
+                        123,
                     )
                 ],
                 [mock.call(123)],
@@ -343,10 +344,10 @@ class TestPollForProLicense:
                 ],
                 [mock.call(mock.ANY, mock.ANY)],
                 [
-                    mock.call(mock.ANY),
-                    mock.call(mock.ANY),
-                    mock.call(mock.ANY),
-                    mock.call(mock.ANY),
+                    mock.call(mock.ANY, mock.ANY),
+                    mock.call(mock.ANY, mock.ANY),
+                    mock.call(mock.ANY, mock.ANY),
+                    mock.call(mock.ANY, mock.ANY),
                 ],
                 [
                     mock.call(123),
