@@ -13,6 +13,7 @@ from uaclient.daemon.retry_auto_attach import (
     retry_auto_attach,
 )
 from uaclient.files import state_files
+from uaclient.testing import fakes
 
 M_PATH = "uaclient.daemon.retry_auto_attach."
 
@@ -22,30 +23,32 @@ class TestFullAutoAttachToFailureReason:
         "e, expected_reason",
         [
             (
-                exceptions.InvalidProImage("invalid pro"),
+                exceptions.InvalidProImage(error_msg="invalid pro"),
                 'Canonical servers did not recognize this machine as Ubuntu Pro: "invalid pro"',  # noqa: E501
             ),
             (
-                exceptions.NonAutoAttachImageError("msg"),
+                exceptions.NonAutoAttachImageError(cloud_type="msg"),
                 "Canonical servers did not recognize this image as Ubuntu Pro",
             ),
             (
-                exceptions.LockHeldError("request", "holder", 123),
+                exceptions.LockHeldError(
+                    lock_request="request", lock_holder="holder", pid=123
+                ),
                 "the pro lock was held by pid 123",
             ),
             (
-                exceptions.ContractAPIError("url", 123, "response"),
+                exceptions.ContractAPIError(
+                    url="url", code=123, body="response"
+                ),
                 'an error from Canonical servers: "response"',
             ),
             (
-                exceptions.ConnectivityError(),
-                "a connectivity error",
-            ),
-            (
-                exceptions.UrlError(error.URLError("urlerror"), "url"),
+                exceptions.ConnectivityError(
+                    cause=error.URLError("urlerror"), url="url"
+                ),
                 'an error while reaching url: "urlerror"',
             ),
-            (exceptions.UserFacingError("msg"), '"msg"'),
+            (fakes.FakeUbuntuProError(), '"This is a test"'),
             (Exception("hello"), "hello"),
             (Exception(), "an unknown error"),
         ],
@@ -523,7 +526,7 @@ class TestRetryAutoAttach:
     ):
         cfg = FakeConfig()
         m_full_auto_attach.side_effect = exceptions.AlreadyAttachedError(
-            "test_account"
+            account_name="test_account"
         )
         retry_auto_attach(cfg)
         assert [mock.call(mock.ANY)] == m_full_auto_attach.call_args_list
@@ -635,7 +638,8 @@ class TestRetryAutoAttach:
                 [mock.call(mock.ANY)],
                 mock.call(
                     AUTO_ATTACH_STATUS_MOTD_FILE,
-                    messages.AUTO_ATTACH_RETRY_TOTAL_FAILURE_NOTICE.format(
+                    "\n"
+                    + messages.AUTO_ATTACH_RETRY_TOTAL_FAILURE_NOTICE.format(
                         num_attempts=19, reason="an unknown error"
                     )
                     + "\n\n",
