@@ -21,7 +21,6 @@ from uaclient.status import ContractStatus
 
 
 class ConcreteTestEntitlement(base.UAEntitlement):
-
     name = "testconcreteentitlement"
     title = "Test Concrete Entitlement"
     description = "Entitlement for testing"
@@ -36,6 +35,8 @@ class ConcreteTestEntitlement(base.UAEntitlement):
         allow_beta=False,
         supports_access_only=False,
         access_only=False,
+        supports_purge=False,
+        purge=False,
         dependent_services=None,
         required_services=None,
         blocking_incompatible_services=None,
@@ -43,11 +44,10 @@ class ConcreteTestEntitlement(base.UAEntitlement):
         **kwargs
     ):
         super().__init__(
-            cfg,
-            allow_beta=allow_beta,
-            access_only=access_only,
+            cfg, allow_beta=allow_beta, access_only=access_only, purge=purge
         )
         self.supports_access_only = supports_access_only
+        self.supports_purge = supports_purge
         self._disable = disable
         self._enable = enable
         self._applicability_status = applicability_status
@@ -105,6 +105,8 @@ def concrete_entitlement_factory(FakeConfig):
         allow_beta: bool = False,
         supports_access_only: bool = False,
         access_only: bool = False,
+        supports_purge: bool = False,
+        purge: bool = False,
         enable: bool = False,
         disable: bool = False,
         dependent_services: Tuple[Any, ...] = None,
@@ -137,6 +139,8 @@ def concrete_entitlement_factory(FakeConfig):
             allow_beta=allow_beta,
             supports_access_only=supports_access_only,
             access_only=access_only,
+            supports_purge=supports_purge,
+            purge=purge,
             enable=enable,
             disable=disable,
             dependent_services=dependent_services,
@@ -150,14 +154,8 @@ def concrete_entitlement_factory(FakeConfig):
 class TestUaEntitlement:
     def test_entitlement_abstract_class(self):
         """UAEntitlement is abstract requiring concrete methods."""
-        with pytest.raises(TypeError) as excinfo:
+        with pytest.raises(TypeError):
             base.UAEntitlement()
-        expected_msg = (
-            "Can't instantiate abstract class UAEntitlement with abstract"
-            " methods _perform_disable, _perform_enable, application_status,"
-            " description, name, title"
-        )
-        assert expected_msg == str(excinfo.value)
 
     def test_init_default_sets_up_uaconfig(self):
         """UAEntitlement sets up a uaconfig instance upon init."""
@@ -323,7 +321,6 @@ class TestUaEntitlementCanEnable:
     def test_can_enable_on_beta_service(
         self, allow_beta_cfg, allow_beta, is_beta, concrete_entitlement_factory
     ):
-
         feature_overrides = {"allow_beta": allow_beta_cfg}
         entitlement = concrete_entitlement_factory(
             entitled=True,
@@ -670,6 +667,27 @@ class TestUaEntitlementEnable:
 
 
 class TestUaEntitlementCanDisable:
+    def test_can_disable_false_on_purge_not_supported(
+        self, concrete_entitlement_factory
+    ):
+        """When the entitlement doesn't support purge, can_disable is FALSE."""
+        entitlement = concrete_entitlement_factory(
+            entitled=True,
+            supports_purge=False,
+            purge=True,
+            application_status=(ApplicationStatus.ENABLED, ""),
+        )
+
+        can_disable, reason = entitlement.can_disable()
+        assert can_disable is False
+        assert reason.reason == CanDisableFailureReason.PURGE_NOT_SUPPORTED
+        assert (
+            reason.message.msg
+            == messages.DISABLE_PURGE_NOT_SUPPORTED.format(
+                title=ConcreteTestEntitlement.title
+            ).msg
+        )
+
     def test_can_disable_false_on_entitlement_inactive(
         self, concrete_entitlement_factory
     ):
@@ -868,7 +886,6 @@ class TestUaEntitlementUserFacingStatus:
     def test_unavailable_when_applicable_but_not_entitled(
         self, concrete_entitlement_factory
     ):
-
         entitlement = concrete_entitlement_factory(
             entitled=False,
             applicability_status=(ApplicabilityStatus.APPLICABLE, ""),
@@ -882,7 +899,6 @@ class TestUaEntitlementUserFacingStatus:
     def test_unavailable_when_applicable_but_no_entitlement_cfg(
         self, concrete_entitlement_factory
     ):
-
         entitlement = concrete_entitlement_factory(
             entitled=False,
             applicability_status=(ApplicabilityStatus.APPLICABLE, ""),
@@ -1253,6 +1269,9 @@ class TestHandleRequiredSnaps:
     @mock.patch(
         "uaclient.entitlements.base.UAEntitlement._base_entitlement_cfg"
     )
+    @mock.patch(
+        "uaclient.snap.is_snapd_installed_as_a_snap", return_value=True
+    )
     @mock.patch("uaclient.snap.is_snapd_installed", return_value=True)
     @mock.patch("uaclient.snap.run_snapd_wait_cmd")
     @mock.patch("uaclient.snap.get_snap_info")
@@ -1267,6 +1286,7 @@ class TestHandleRequiredSnaps:
         m_get_snap_info,
         m_run_snapd_wait_cmd,
         m_is_snapd_installed,
+        m_is_snapd_installed_as_a_snap,
         m_base_ent_cfg,
         entitlement_cfg,
         concrete_entitlement_factory,
@@ -1274,8 +1294,8 @@ class TestHandleRequiredSnaps:
         entitlement = concrete_entitlement_factory()
         m_base_ent_cfg.return_value = entitlement_cfg
         m_get_snap_info.side_effect = [
-            exceptions.SnapNotInstalledError("snap"),
-            exceptions.SnapNotInstalledError("snap"),
+            exceptions.SnapNotInstalledError(snap="snap"),
+            exceptions.SnapNotInstalledError(snap="snap"),
             mock.MagicMock,
         ]
 
@@ -1283,11 +1303,13 @@ class TestHandleRequiredSnaps:
 
         if not entitlement_cfg:
             assert 0 == m_is_snapd_installed.call_count
+            assert 0 == m_is_snapd_installed_as_a_snap.call_count
             assert 0 == m_run_snapd_wait_cmd.call_count
             assert 0 == m_validate_proxy.call_count
             assert 0 == m_configure_snap_proxy.call_count
         else:
             assert 1 == m_is_snapd_installed.call_count
+            assert 1 == m_is_snapd_installed_as_a_snap.call_count
             assert 1 == m_run_snapd_wait_cmd.call_count
             assert 2 == m_validate_proxy.call_count
             assert 1 == m_configure_snap_proxy.call_count
@@ -1339,7 +1361,9 @@ class TestHandleRequiredPackages:
         ],
     )
     @mock.patch("uaclient.apt.run_apt_install_command")
-    @mock.patch("uaclient.apt.run_apt_update_command")
+    @mock.patch(
+        "uaclient.entitlements.base.UAEntitlement._update_sources_list"
+    )
     @mock.patch(
         "uaclient.entitlements.base.UAEntitlement.entitlement_cfg",
         new_callable=mock.PropertyMock,
@@ -1347,7 +1371,7 @@ class TestHandleRequiredPackages:
     def test_handle_required_packages(
         self,
         m_entitlement_cfg,
-        m_apt_update,
+        m_update_sources_list,
         m_apt_install,
         required_packages_directive,
         expected_apt_update_calls,
@@ -1363,7 +1387,9 @@ class TestHandleRequiredPackages:
         }
 
         assert expected_result == entitlement.handle_required_packages()
-        assert expected_apt_update_calls == m_apt_update.call_args_list
+        assert (
+            expected_apt_update_calls == m_update_sources_list.call_args_list
+        )
         assert expected_apt_install_calls == m_apt_install.call_args_list
 
     @pytest.mark.parametrize(
@@ -1385,12 +1411,12 @@ class TestHandleRequiredPackages:
             ),
             (
                 [{"name": "package"}],
-                [mock.call([], mock.ANY)],
+                [],
                 True,
             ),
             (
                 [{"name": "package"}, {"name": "package2"}],
-                [mock.call([], mock.ANY)],
+                [],
                 True,
             ),
             (
