@@ -81,7 +81,9 @@ class TestEntitlementFactory:
                 )
 
         assert (
-            messages.ENTITLEMENT_NOT_FOUND.format(name="nonexistent").msg
+            messages.E_ENTITLEMENT_NOT_FOUND.format(
+                entitlement_name="nonexistent"
+            ).msg
             == excinfo.value.msg
         )
 
