@@ -11,7 +11,7 @@ import mock
 import pytest
 
 import uaclient.entitlements.fips as fips
-from uaclient import apt, defaults, exceptions, messages, system, util
+from uaclient import apt, exceptions, messages, system, util
 from uaclient.clouds.identity import NoCloudTypeReason
 from uaclient.entitlements.entitlement_status import (
     ApplicabilityStatus,
@@ -28,6 +28,7 @@ from uaclient.entitlements.fips import (
     FIPSUpdatesEntitlement,
 )
 from uaclient.files.notices import Notice, NoticesManager
+from uaclient.testing import fakes
 
 M_PATH = "uaclient.entitlements.fips."
 M_LIVEPATCH_PATH = "uaclient.entitlements.livepatch.LivepatchEntitlement."
@@ -140,7 +141,9 @@ class TestFIPSEntitlementDefaults:
                         util.prompt_for_confirmation,
                         {
                             "assume_yes": assume_yes,
-                            "msg": messages.PROMPT_FIPS_PRE_DISABLE,
+                            "msg": messages.PROMPT_FIPS_PRE_DISABLE.format(
+                                title="FIPS"
+                            ),
                         },
                     )
                 ],
@@ -161,7 +164,9 @@ class TestFIPSEntitlementDefaults:
                         util.prompt_for_confirmation,
                         {
                             "assume_yes": assume_yes,
-                            "msg": messages.PROMPT_FIPS_PRE_DISABLE,
+                            "msg": messages.PROMPT_FIPS_PRE_DISABLE.format(
+                                title="FIPS Updates",
+                            ),
                         },
                     )
                 ],
@@ -199,7 +204,9 @@ class TestFIPSEntitlementDefaults:
                         util.prompt_for_confirmation,
                         {
                             "assume_yes": False,
-                            "msg": messages.PROMPT_FIPS_PRE_DISABLE,
+                            "msg": messages.PROMPT_FIPS_PRE_DISABLE.format(
+                                title="FIPS"
+                            ),
                         },
                     )
                 ],
@@ -222,7 +229,9 @@ class TestFIPSEntitlementDefaults:
                         util.prompt_for_confirmation,
                         {
                             "assume_yes": False,
-                            "msg": messages.PROMPT_FIPS_PRE_DISABLE,
+                            "msg": messages.PROMPT_FIPS_PRE_DISABLE.format(
+                                title="FIPS Updates"
+                            ),
                         },
                     )
                 ],
@@ -261,12 +270,14 @@ class TestFIPSEntitlementCanEnable:
 
 
 class TestFIPSEntitlementEnable:
+    @mock.patch("uaclient.apt.update_sources_list")
     @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch(M_PATH + "get_cloud_type", return_value=("", None))
     def test_enable_configures_apt_sources_and_auth_files(
         self,
         _m_get_cloud_type,
         m_setup_apt_proxy,
+        m_update_sources_list,
         entitlement,
     ):
         """When entitled, configure apt repo auth token, pinning and url."""
@@ -394,12 +405,6 @@ class TestFIPSEntitlementEnable:
                 retry_sleeps=apt.APT_RETRIES,
                 override_env_vars=None,
             ),
-            mock.call(
-                ["apt-get", "update"],
-                capture=True,
-                retry_sleeps=apt.APT_RETRIES,
-                override_env_vars=None,
-            ),
         ]
         subp_calls += install_cmd
 
@@ -409,8 +414,9 @@ class TestFIPSEntitlementEnable:
         assert add_apt_calls == m_add_apt.call_args_list
         assert apt_pinning_calls == m_add_pinning.call_args_list
         assert subp_calls == m_subp.call_args_list
+        assert 2 == m_update_sources_list.call_count
         assert [
-            messages.FIPS_SYSTEM_REBOOT_REQUIRED.msg,
+            messages.FIPS_SYSTEM_REBOOT_REQUIRED,
         ] == notice_ent_cls.list()
 
     @pytest.mark.parametrize(
@@ -494,10 +500,12 @@ class TestFIPSEntitlementEnable:
             entitlement, "can_enable", return_value=(True, None)
         ):
             with mock.patch("uaclient.util.handle_message_operations"):
-                with pytest.raises(exceptions.UserFacingError) as excinfo:
+                with pytest.raises(exceptions.UbuntuProError) as excinfo:
                     entitlement.enable()
-        error_msg = "Empty {} apt suites directive from {}".format(
-            entitlement.name, defaults.BASE_CONTRACT_URL
+        error_msg = (
+            "Ubuntu Pro server provided no suites directive for {}".format(
+                entitlement.name
+            )
         )
         assert error_msg == excinfo.value.msg
         assert 0 == m_add_apt.call_count
@@ -535,12 +543,15 @@ class TestFIPSEntitlementEnable:
             )
             stack.enter_context(mock.patch(M_REPOPATH + "exists"))
 
-            with pytest.raises(exceptions.UserFacingError) as excinfo:
+            with pytest.raises(exceptions.UbuntuProError) as excinfo:
                 entitlement.enable()
 
         error_msg = (
-            "Cannot setup apt pin. Empty apt repo origin value 'None'.\n"
-            "Could not enable {}.".format(entitlement.title)
+            "Cannot setup apt pin. Empty apt repo origin value for {}\n"
+            "Could not enable {}."
+        ).format(
+            entitlement.name,
+            entitlement.title,
         )
         assert error_msg == excinfo.value.msg
         assert 0 == m_add_apt.call_count
@@ -873,9 +884,12 @@ class TestFIPSEntitlementRemovePackages:
     ):
         m_get_installed_packages.return_value = ["ubuntu-fips"]
         m_subp.side_effect = exceptions.ProcessExecutionError(cmd="test")
-        expected_msg = "Could not disable {}.".format(entitlement.title)
+        expected_msg = (
+            "Unexpected APT error.\nCould not disable {}.\nSee "
+            "/var/log/ubuntu-advantage.log"
+        ).format(entitlement.title)
 
-        with pytest.raises(exceptions.UserFacingError) as exc_info:
+        with pytest.raises(exceptions.UbuntuProError) as exc_info:
             entitlement.remove_packages()
 
         assert exc_info.value.msg.strip() == expected_msg
@@ -981,7 +995,7 @@ class TestFIPSEntitlementApplicationStatus:
         msg = messages.NamedMessage("test-code", "sure is some status here")
         notice_ent_cls.add(
             Notice.FIPS_SYSTEM_REBOOT_REQUIRED,
-            messages.FIPS_SYSTEM_REBOOT_REQUIRED.msg,
+            messages.FIPS_SYSTEM_REBOOT_REQUIRED,
         )
 
         if path_exists:
@@ -1014,7 +1028,7 @@ class TestFIPSEntitlementApplicationStatus:
         if path_exists and should_reboot and proc_content == "1":
             expected_msg = msg
             assert [
-                messages.FIPS_SYSTEM_REBOOT_REQUIRED.msg,
+                messages.FIPS_SYSTEM_REBOOT_REQUIRED,
                 messages.FIPS_REBOOT_REQUIRED_MSG,
             ] == notice_ent_cls.list()
         elif path_exists and not should_reboot and proc_content == "1":
@@ -1030,7 +1044,7 @@ class TestFIPSEntitlementApplicationStatus:
             assert [
                 messages.FIPS_DISABLE_REBOOT_REQUIRED,
                 messages.NOTICE_FIPS_MANUAL_DISABLE_URL,
-                messages.FIPS_SYSTEM_REBOOT_REQUIRED.msg,
+                messages.FIPS_SYSTEM_REBOOT_REQUIRED,
                 messages.FIPS_REBOOT_REQUIRED_MSG,
             ] == notice_ent_cls.list()
         elif path_exists and not should_reboot and proc_content == "0":
@@ -1076,9 +1090,9 @@ class TestFipsEntitlementInstallPackages:
     def test_install_packages_fail_if_metapackage_not_installed(
         self, m_run_apt, entitlement
     ):
-        m_run_apt.side_effect = exceptions.UserFacingError("error")
+        m_run_apt.side_effect = fakes.FakeUbuntuProError()
         with mock.patch.object(entitlement, "remove_apt_config"):
-            with pytest.raises(exceptions.UserFacingError):
+            with pytest.raises(exceptions.UbuntuProError):
                 entitlement.install_packages()
 
     @mock.patch(M_PATH + "apt.get_installed_packages_names")
@@ -1098,8 +1112,8 @@ class TestFipsEntitlementInstallPackages:
 
         m_run_apt_install.side_effect = [
             True,
-            exceptions.UserFacingError("error"),
-            exceptions.UserFacingError("error"),
+            fakes.FakeUbuntuProError(),
+            fakes.FakeUbuntuProError(),
         ]
 
         fake_stdout = io.StringIO()
@@ -1120,7 +1134,6 @@ class TestFipsEntitlementInstallPackages:
                         '-o Dpkg::Options::="--force-confdef"',
                         '-o Dpkg::Options::="--force-confold"',
                     ],
-                    error_msg="Could not enable {}.".format(entitlement.title),
                     override_env_vars={"DEBIAN_FRONTEND": "noninteractive"},
                 )
             )
@@ -1128,9 +1141,12 @@ class TestFipsEntitlementInstallPackages:
         expected_msg = "\n".join(
             [
                 "Installing {} packages".format(entitlement.title),
+                "Updating standard Ubuntu package lists",
+                "Could not enable {}.".format(entitlement.title),
                 messages.FIPS_PACKAGE_NOT_AVAILABLE.format(
                     service=entitlement.title, pkg="b"
                 ),
+                "Could not enable {}.".format(entitlement.title),
                 messages.FIPS_PACKAGE_NOT_AVAILABLE.format(
                     service=entitlement.title, pkg="c"
                 ),
@@ -1171,11 +1187,23 @@ class TestFipsSetupAPTConfig:
         run_apt_command.return_value = held_packages
         entitlement.setup_apt_config(silent=False)
         expected_calls = [
-            mock.call(["apt-mark", "showholds"], "apt-mark showholds failed.")
+            mock.call(
+                ["apt-mark", "showholds"],
+                messages.EXECUTING_COMMAND_FAILED.format(
+                    command="apt-mark showholds"
+                ),
+            )
         ]
         if unhold_packages:
             cmd = ["apt-mark", "unhold"] + unhold_packages
-            expected_calls.append(mock.call(cmd, " ".join(cmd) + " failed."))
+            expected_calls.append(
+                mock.call(
+                    cmd,
+                    messages.EXECUTING_COMMAND_FAILED.format(
+                        command=" ".join(cmd)
+                    ),
+                )
+            )
         assert expected_calls == run_apt_command.call_args_list
         assert [mock.call(silent=False)] == setup_apt_config.call_args_list
 
