@@ -23,6 +23,7 @@ from uaclient.entitlements.livepatch import (
 )
 from uaclient.entitlements.tests.conftest import machine_token
 from uaclient.snap import SNAP_CMD
+from uaclient.testing import fakes
 
 M_PATH = "uaclient.entitlements.livepatch."  # mock path
 M_LIVEPATCH_STATUS = M_PATH + "LivepatchEntitlement.application_status"
@@ -371,24 +372,38 @@ class TestLivepatchProcessContractDeltas:
         assert setup_calls == m_setup_livepatch_config.call_args_list
 
 
+@mock.patch(M_PATH + "snap.is_snapd_installed_as_a_snap")
 @mock.patch(M_PATH + "snap.is_snapd_installed")
 @mock.patch("uaclient.http.validate_proxy", side_effect=lambda x, y, z: y)
 @mock.patch("uaclient.snap.configure_snap_proxy")
 @mock.patch("uaclient.livepatch.configure_livepatch_proxy")
 class TestLivepatchEntitlementEnable:
 
-    mocks_apt_update = [mock.call()]
+    mocks_apt_update = []
     mocks_snapd_install = [
         mock.call(
             ["apt-get", "install", "--assume-yes", "snapd"],
             retry_sleeps=apt.APT_RETRIES,
         )
     ]
+    mocks_snapd_install_as_a_snap = [
+        mock.call(
+            ["/usr/bin/snap", "install", "snapd"],
+            capture=True,
+            retry_sleeps=[0.5, 1, 5],
+        )
+    ]
     mocks_snap_wait_seed = [
         mock.call(
             ["/usr/bin/snap", "wait", "system", "seed.loaded"], capture=True
         )
     ]
+    mocks_snapd_refresh = [
+        mock.call(
+            ["/usr/bin/snap", "refresh", "snapd"],
+            capture=True,
+        )
+    ]
     mocks_livepatch_install = [
         mock.call(
             ["/usr/bin/snap", "install", "canonical-livepatch"],
@@ -397,7 +412,11 @@ class TestLivepatchEntitlementEnable:
         )
     ]
     mocks_install = (
-        mocks_snapd_install + mocks_snap_wait_seed + mocks_livepatch_install
+        mocks_snapd_install
+        + mocks_snapd_install_as_a_snap
+        + mocks_snap_wait_seed
+        + mocks_snapd_refresh
+        + mocks_livepatch_install
     )
     mocks_config = [
         mock.call(
@@ -420,6 +439,7 @@ class TestLivepatchEntitlementEnable:
     @mock.patch("uaclient.system.get_release_info")
     @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.contract.apply_contract_overrides")
+    @mock.patch("uaclient.apt.update_sources_list")
     @mock.patch("uaclient.apt.run_apt_install_command")
     @mock.patch("uaclient.apt.run_apt_update_command")
     @mock.patch("uaclient.system.which", return_value=None)
@@ -434,6 +454,7 @@ class TestLivepatchEntitlementEnable:
         m_which,
         m_run_apt_update,
         m_run_apt_install,
+        m_update_sources_list,
         _m_contract_overrides,
         m_subp,
         _m_get_release_info,
@@ -441,6 +462,7 @@ class TestLivepatchEntitlementEnable:
         m_snap_proxy,
         m_validate_proxy,
         m_is_snapd_installed,
+        m_is_snapd_installed_as_a_snap,
         capsys,
         caplog_text,
         event,
@@ -451,27 +473,31 @@ class TestLivepatchEntitlementEnable:
         application_status = ApplicationStatus.ENABLED
         m_app_status.return_value = application_status, "enabled"
         m_is_snapd_installed.return_value = False
+        m_is_snapd_installed_as_a_snap.return_value = False
 
-        def fake_run_apt_update():
+        def fake_update_sources_list(sources_list):
             if apt_update_success:
                 return
-            raise exceptions.UserFacingError("Apt go BOOM")
+            raise fakes.FakeUbuntuProError()
 
-        m_run_apt_update.side_effect = fake_run_apt_update
+        m_update_sources_list.side_effect = fake_update_sources_list
 
         assert entitlement.enable()
         assert self.mocks_install + self.mocks_config in m_subp.call_args_list
         assert self.mocks_apt_update == m_run_apt_update.call_args_list
+        assert 1 == m_update_sources_list.call_count
         msg = (
             "Installing snapd\n"
-            "Updating package lists\n"
+            "Updating standard Ubuntu package lists\n"
+            "Installing snapd snap\n"
             "Installing canonical-livepatch snap\n"
-            "Canonical livepatch enabled.\n"
+            "Disabling Livepatch prior to re-attach with new token\n"
+            "Canonical Livepatch enabled\n"
         )
         assert (msg, "") == capsys.readouterr()
         expected_log = (
             "DEBUG    Trying to install snapd."
-            " Ignoring apt-get update failure: Apt go BOOM"
+            " Ignoring apt-get update failure: This is a test"
         )
         if apt_update_success:
             assert expected_log not in caplog_text()
@@ -482,6 +508,74 @@ class TestLivepatchEntitlementEnable:
         assert m_snap_proxy.call_count == 1
         assert m_livepatch_proxy.call_count == 1
 
+    @pytest.mark.parametrize("caplog_text", [logging.DEBUG], indirect=True)
+    @mock.patch("uaclient.system.get_release_info")
+    @mock.patch("uaclient.system.subp")
+    @mock.patch("uaclient.contract.apply_contract_overrides")
+    @mock.patch("uaclient.apt.update_sources_list")
+    @mock.patch("uaclient.apt.run_apt_install_command")
+    @mock.patch("uaclient.apt.run_apt_update_command")
+    @mock.patch("uaclient.system.which", return_value=None)
+    @mock.patch(M_PATH + "LivepatchEntitlement.application_status")
+    @mock.patch(
+        M_PATH + "LivepatchEntitlement.can_enable", return_value=(True, None)
+    )
+    def test_enable_continues_when_snap_install_snapd_fails(
+        self,
+        m_can_enable,
+        m_app_status,
+        m_which,
+        m_run_apt_update,
+        m_run_apt_install,
+        m_update_sources_list,
+        _m_contract_overrides,
+        m_subp,
+        _m_get_release_info,
+        m_livepatch_proxy,
+        m_snap_proxy,
+        m_validate_proxy,
+        m_is_snapd_installed,
+        m_is_snapd_installed_as_a_snap,
+        capsys,
+        caplog_text,
+        event,
+        entitlement,
+    ):
+        """Install snapd and canonical-livepatch snap when not on system."""
+        application_status = ApplicationStatus.ENABLED
+        m_app_status.return_value = application_status, "enabled"
+        m_is_snapd_installed.return_value = False
+        m_is_snapd_installed_as_a_snap.return_value = False
+        m_subp.side_effect = [
+            None,
+            exceptions.ProcessExecutionError("test"),
+            None,
+            None,
+            None,
+            None,
+            None,
+            None,
+        ]
+
+        assert entitlement.enable()
+        assert self.mocks_install + self.mocks_config in m_subp.call_args_list
+        assert self.mocks_apt_update == m_run_apt_update.call_args_list
+        assert 1 == m_update_sources_list.call_count
+        msg = (
+            "Installing snapd\n"
+            "Updating standard Ubuntu package lists\n"
+            "Installing snapd snap\n"
+            "Executing `snap install snapd` failed.\n"
+            "Installing canonical-livepatch snap\n"
+            "Disabling Livepatch prior to re-attach with new token\n"
+            "Canonical Livepatch enabled\n"
+        )
+        assert (msg, "") == capsys.readouterr()
+        assert [mock.call(livepatch.LIVEPATCH_CMD)] == m_which.call_args_list
+        assert m_validate_proxy.call_count == 2
+        assert m_snap_proxy.call_count == 1
+        assert m_livepatch_proxy.call_count == 1
+
     @mock.patch("uaclient.system.get_release_info")
     @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.contract.apply_contract_overrides")
@@ -502,6 +596,7 @@ class TestLivepatchEntitlementEnable:
         m_snap_proxy,
         m_validate_proxy,
         m_is_snapd_installed,
+        m_is_snapd_installed_as_a_snap,
         capsys,
         event,
         entitlement,
@@ -514,13 +609,15 @@ class TestLivepatchEntitlementEnable:
         assert entitlement.enable()
         assert (
             self.mocks_snap_wait_seed
+            + self.mocks_snapd_refresh
             + self.mocks_livepatch_install
             + self.mocks_config
             in m_subp.call_args_list
         )
         msg = (
             "Installing canonical-livepatch snap\n"
-            "Canonical livepatch enabled.\n"
+            "Disabling Livepatch prior to re-attach with new token\n"
+            "Canonical Livepatch enabled\n"
         )
         assert (msg, "") == capsys.readouterr()
         assert [mock.call(livepatch.LIVEPATCH_CMD)] == m_which.call_args_list
@@ -550,6 +647,7 @@ class TestLivepatchEntitlementEnable:
         m_snap_proxy,
         m_validate_proxy,
         m_is_snapd_installed,
+        m_is_snapd_installed_as_a_snap,
         capsys,
         event,
         entitlement,
@@ -564,6 +662,7 @@ class TestLivepatchEntitlementEnable:
             mock.call(
                 [SNAP_CMD, "wait", "system", "seed.loaded"], capture=True
             ),
+            mock.call([SNAP_CMD, "refresh", "snapd"], capture=True),
             mock.call(
                 [
                     livepatch.LIVEPATCH_CMD,
@@ -579,7 +678,11 @@ class TestLivepatchEntitlementEnable:
             ),
         ]
         assert subp_calls == m_subp.call_args_list
-        assert ("Canonical livepatch enabled.\n", "") == capsys.readouterr()
+        assert (
+            "Disabling Livepatch prior to re-attach with new token\n"
+            "Canonical Livepatch enabled\n",
+            "",
+        ) == capsys.readouterr()
         assert m_validate_proxy.call_count == 2
         assert m_snap_proxy.call_count == 1
         assert m_livepatch_proxy.call_count == 1
@@ -606,6 +709,7 @@ class TestLivepatchEntitlementEnable:
         m_snap_proxy,
         m_validate_proxy,
         m_is_snapd_installed,
+        m_is_snapd_installed_as_a_snap,
         capsys,
         entitlement,
     ):
@@ -618,6 +722,7 @@ class TestLivepatchEntitlementEnable:
             mock.call(
                 [SNAP_CMD, "wait", "system", "seed.loaded"], capture=True
             ),
+            mock.call([SNAP_CMD, "refresh", "snapd"], capture=True),
             mock.call(
                 [
                     livepatch.LIVEPATCH_CMD,
@@ -632,7 +737,7 @@ class TestLivepatchEntitlementEnable:
             ),
         ]
         assert subp_no_livepatch_disable == m_subp.call_args_list
-        assert ("Canonical livepatch enabled.\n", "") == capsys.readouterr()
+        assert ("Canonical Livepatch enabled\n", "") == capsys.readouterr()
         assert m_validate_proxy.call_count == 2
         assert m_snap_proxy.call_count == 1
         assert m_livepatch_proxy.call_count == 1
@@ -650,6 +755,7 @@ class TestLivepatchEntitlementEnable:
         m_snap_proxy,
         m_validate_proxy,
         _m_is_snapd_installed,
+        m_is_snapd_installed_as_a_snap,
         cls_name,
         cls_title,
         entitlement,
@@ -686,6 +792,7 @@ class TestLivepatchEntitlementEnable:
         m_snap_proxy,
         m_validate_proxy,
         m_is_snapd_installed,
+        m_is_snapd_installed_as_a_snap,
         entitlement,
         capsys,
         caplog_text,
@@ -708,6 +815,7 @@ class TestLivepatchEntitlementEnable:
                 stderr=stderr_msg,
             ),
             True,
+            True,
         ]
 
         fake_stdout = io.StringIO()
@@ -728,8 +836,10 @@ class TestLivepatchEntitlementEnable:
             in fake_stdout.getvalue().strip()
         )
 
-        for msg in messages.SNAPD_DOES_NOT_HAVE_WAIT_CMD.split("\n"):
-            assert msg in caplog_text()
+        assert (
+            "Detected version of snapd that does not have wait command"
+            in caplog_text()
+        )
 
         assert m_validate_proxy.call_count == 2
         assert m_snap_proxy.call_count == 1
@@ -747,6 +857,7 @@ class TestLivepatchEntitlementEnable:
         m_snap_proxy,
         m_validate_proxy,
         m_is_snapd_installed,
+        m_is_snapd_installed_as_a_snap,
         entitlement,
     ):
         m_is_snapd_installed.return_value = False
@@ -782,6 +893,7 @@ class TestLivepatchEntitlementEnable:
         m_snap_proxy,
         m_validate_proxy,
         m_is_snapd_installed,
+        m_is_snapd_installed_as_a_snap,
         entitlement,
     ):
         m_is_snapd_installed.return_value = True
@@ -853,3 +965,19 @@ class TestLivepatchApplicationStatus:
         else:
             assert status == ApplicationStatus.ENABLED
             assert details is None
+
+    @mock.patch("uaclient.livepatch.is_livepatch_installed", return_value=True)
+    @mock.patch("uaclient.livepatch.status")
+    def test_application_status_when_canonical_livepatch_fails(
+        self, m_status, _m_livepatch_installed, entitlement
+    ):
+        m_status.side_effect = exceptions.ProcessExecutionError(
+            cmd="test", stdout="", stderr="livepatch error"
+        )
+
+        status, details = entitlement.application_status()
+
+        assert status == ApplicationStatus.WARNING
+        assert details == messages.LIVEPATCH_CLIENT_FAILURE_WARNING.format(
+            livepatch_error="livepatch error"
+        )
