@@ -244,6 +244,9 @@ class TestProcessContractDeltas:
         assert [] == m_remove_auth_apt_repo.call_args_list
         assert 1 == m_check_apt_url_applied.call_count
 
+    @pytest.mark.parametrize(
+        "series,file_extension", (("jammy", "list"), ("noble", "sources"))
+    )
     @mock.patch(
         "uaclient.entitlements.base.UAEntitlement.process_contract_deltas"
     )
@@ -262,6 +265,8 @@ class TestProcessContractDeltas:
         m_release_info,
         m_read_cache,
         m_process_contract_deltas,
+        series,
+        file_extension,
         entitlement,
     ):
         """Remove old apt url when aptURL delta occurs on active service."""
@@ -270,6 +275,7 @@ class TestProcessContractDeltas:
         m_read_cache.return_value = {
             "services": [{"name": "repotest", "status": "enabled"}]
         }
+        m_release_info.return_value.series = series
         assert entitlement.process_contract_deltas(
             {
                 "entitlement": {
@@ -289,7 +295,10 @@ class TestProcessContractDeltas:
         assert [mock.call()] == m_setup_apt_config.call_args_list
         apt_auth_remove_calls = [
             mock.call(
-                "/etc/apt/sources.list.d/ubuntu-repotest.list", "http://old"
+                "/etc/apt/sources.list.d/ubuntu-repotest.{}".format(
+                    file_extension
+                ),
+                "http://old",
             )
         ]
         assert apt_auth_remove_calls == m_remove_auth_apt_repo.call_args_list
@@ -421,6 +430,10 @@ class TestRepoEnable:
     @pytest.mark.parametrize("should_reboot", (False, True))
     @pytest.mark.parametrize("with_pre_install_msg", (False, True))
     @pytest.mark.parametrize("packages", (["a"], [], None))
+    @pytest.mark.parametrize(
+        "series,file_extension", (("xenial", "list"), ("noble", "sources"))
+    )
+    @mock.patch("uaclient.apt.update_sources_list")
     @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch(M_PATH + "system.should_reboot")
     @mock.patch(M_PATH + "system.subp", return_value=("", ""))
@@ -439,16 +452,19 @@ class TestRepoEnable:
         m_subp,
         m_should_reboot,
         m_setup_apt_proxy,
+        m_update_sources_list,
         entitlement,
         capsys,
         caplog_text,
         event,
         packages,
+        series,
+        file_extension,
         with_pre_install_msg,
         should_reboot,
     ):
         """On enable add authenticated apt repo and refresh package lists."""
-        m_release_info.return_value = mock.MagicMock(series="xenial")
+        m_release_info.return_value = mock.MagicMock(series=series)
         m_should_reboot.return_value = should_reboot
 
         pre_install_msgs = ["Some pre-install information", "Some more info"]
@@ -461,26 +477,24 @@ class TestRepoEnable:
         else:
             messaging_patch = mock.MagicMock()
 
-        expected_apt_calls = [
-            mock.call(
-                ["apt-get", "update"],
-                capture=True,
-                retry_sleeps=apt.APT_RETRIES,
-                override_env_vars=None,
-            )
-        ]
+        expected_apt_calls = []
 
         reboot_msg = "A reboot is required to complete install."
         expected_output = (
             "\n".join(
-                ["Updating package lists", "Repo Test Class enabled"]
+                [
+                    "Updating Repo Test Class package lists",
+                    "Repo Test Class enabled",
+                ]
                 + ([reboot_msg] if should_reboot else [])
             )
             + "\n"
         )
 
+        update_sources_list_call_count = 1
         if packages is not None:
             if len(packages) > 0:
+                update_sources_list_call_count += 1
                 expected_apt_calls.append(
                     mock.call(
                         [
@@ -496,8 +510,9 @@ class TestRepoEnable:
                 )
                 expected_output = (
                     "\n".join(
-                        ["Updating package lists"]
+                        ["Updating Repo Test Class package lists"]
                         + (pre_install_msgs if with_pre_install_msg else [])
+                        + ["Updating standard Ubuntu package lists"]
                         + [
                             "Installing Repo Test Class packages",
                             "Repo Test Class enabled",
@@ -520,9 +535,14 @@ class TestRepoEnable:
         ]
         assert expected_calls in m_exists.call_args_list
         assert expected_apt_calls == m_subp.call_args_list
+        assert (
+            update_sources_list_call_count == m_update_sources_list.call_count
+        )
         add_apt_calls = [
             mock.call(
-                "/etc/apt/sources.list.d/ubuntu-repotest.list",
+                "/etc/apt/sources.list.d/ubuntu-repotest.{}".format(
+                    file_extension
+                ),
                 "http://REPOTEST/ubuntu",
                 "repotest-token",
                 ["xenial"],
@@ -554,13 +574,18 @@ class TestRepoEnable:
                 with mock.patch.object(
                     type(entitlement), "packages", packages
                 ):
-                    with pytest.raises(exceptions.UserFacingError) as excinfo:
+                    with pytest.raises(exceptions.UbuntuProError) as excinfo:
                         with mock.patch.object(
                             entitlement, "remove_apt_config"
                         ) as m_rac:
                             entitlement.enable()
 
-        assert "Could not enable Repo Test Class." == excinfo.value.msg
+        assert (
+            "Unexpected APT error.\n"
+            "Invalid command specified '['apt-get', 'install', "
+            "'--assume-yes', 'fake_pkg', 'and_another']'.\n"
+            "See /var/log/ubuntu-advantage.log"
+        ) == excinfo.value.msg
         assert 1 == m_rac.call_count
 
 
@@ -638,6 +663,285 @@ class TestPerformEnable:
             )
 
 
+class TestPerformDisable:
+    @pytest.mark.parametrize("purge_value", (True, False))
+    @mock.patch(M_PATH + "apt.get_installed_packages_by_origin")
+    @mock.patch(M_PATH + "apt.get_remote_versions_for_package")
+    @mock.patch(M_PATH + "RepoEntitlement.prompt_for_purge")
+    @mock.patch(M_PATH + "RepoEntitlement.execute_removal")
+    @mock.patch(M_PATH + "RepoEntitlement.execute_reinstall")
+    @mock.patch(M_PATH + "RepoEntitlement.remove_apt_config")
+    def test_purge_functions_are_called(
+        self,
+        m_remove_apt_config,
+        m_execute_reinstall,
+        m_execute_removal,
+        m_prompt_for_purge,
+        m_get_remote_versions,
+        m_get_installed_packages,
+        purge_value,
+        entitlement_factory,
+    ):
+        m_get_installed_packages.return_value = [1, 2, 3, 4, 5]
+
+        def return_alternatives(p, _exclude_origins):
+            if p % 2:
+                return [p]
+            return []
+
+        m_get_remote_versions.side_effect = return_alternatives
+
+        with mock.patch.object(RepoTestEntitlement, "origin", "TestOrigin"):
+            entitlement = entitlement_factory(
+                RepoTestEntitlement,
+                affordances={"series": ["xenial"]},
+                purge=purge_value,
+            )
+            assert entitlement._perform_disable() is True
+            assert m_remove_apt_config.call_args_list == [
+                mock.call(silent=mock.ANY)
+            ]
+
+            if purge_value:
+                m_get_installed_packages.call_args_list = [
+                    mock.call("TestOrigin")
+                ]
+                m_get_remote_versions.call_args_list = [
+                    mock.call(1, exclude_origin="TestOrigin"),
+                    mock.call(2, exclude_origin="TestOrigin"),
+                    mock.call(3, exclude_origin="TestOrigin"),
+                    mock.call(4, exclude_origin="TestOrigin"),
+                    mock.call(5, exclude_origin="TestOrigin"),
+                ]
+                m_prompt_for_purge.call_args_list = [
+                    mock.call([1, 3, 5], [(2, 2), (4, 4)])
+                ]
+                m_execute_removal.call_args_list = [mock.call([1, 3, 5])]
+                m_execute_reinstall.call_args_list = [
+                    mock.call([(2, 2), (4, 4)])
+                ]
+            else:
+                m_get_installed_packages.call_args_list = []
+                m_get_remote_versions.call_args_list = []
+                m_prompt_for_purge.call_args_list = []
+                m_execute_removal.call_args_list = []
+                m_execute_reinstall.call_args_list = []
+
+
+class TestPurge:
+    # This circular import should NOT exist, test_apt needs a refactor.
+    from uaclient.tests.test_apt import mock_package, mock_version
+
+    packages_to_remove = [
+        mock_package("remove1", mock_version("1.0")),
+        mock_package("remove2", mock_version("1.0")),
+    ]
+    packages_to_reinstall = [
+        (mock_package("reinstall1", mock_version("1.0")), mock_version("2.0")),
+        (mock_package("reinstall2", mock_version("1.0")), mock_version("2.0")),
+    ]
+    mock_kernel_package = [mock_package("linux-image-1.2.3-4-fake")]
+
+    def test_purge_kernel_check_true_if_no_kernel(self, entitlement_factory):
+        entitlement = entitlement_factory(
+            RepoTestEntitlement,
+            affordances={"series": ["xenial"]},
+            purge=True,
+        )
+
+        assert entitlement.purge_kernel_check(self.packages_to_remove) is True
+
+    @mock.patch(
+        M_PATH + "system.get_installed_ubuntu_kernels", return_value=[]
+    )
+    @mock.patch(M_PATH + "system.get_kernel_info")
+    def test_purge_kernel_check_false_when_no_other_kernels(
+        self, _m_kernel_info, _m_installed_kernels, entitlement_factory, capsys
+    ):
+        entitlement = entitlement_factory(
+            RepoTestEntitlement,
+            affordances={"series": ["xenial"]},
+            purge=True,
+        )
+
+        assert (
+            entitlement.purge_kernel_check(self.mock_kernel_package) is False
+        )
+        out, _err = capsys.readouterr()
+        assert "No other valid Ubuntu kernel was found in the system" in out
+
+    @pytest.mark.parametrize("prompt_answer", (True, False))
+    @pytest.mark.parametrize(
+        "current_kernel", ("1.2.3-4-fake", "2.3.4-5-fake")
+    )
+    @mock.patch(M_PATH + "util.prompt_for_confirmation")
+    @mock.patch(
+        M_PATH + "system.get_installed_ubuntu_kernels",
+        return_value=["2.3.4-5-fake"],
+    )
+    @mock.patch(M_PATH + "system.get_kernel_info")
+    def test_purge_kernel_check_prompts_if_other_kernels(
+        self,
+        m_kernel_info,
+        _m_installed_kernels,
+        m_confirmation,
+        prompt_answer,
+        current_kernel,
+        entitlement_factory,
+        capsys,
+    ):
+        m_kernel_info.return_value.uname_release = current_kernel
+        m_confirmation.return_value = prompt_answer
+        entitlement = entitlement_factory(
+            RepoTestEntitlement,
+            affordances={"series": ["xenial"]},
+            purge=True,
+        )
+
+        assert (
+            entitlement.purge_kernel_check(self.mock_kernel_package)
+            is prompt_answer
+        )
+
+        out, _err = capsys.readouterr()
+        assert "would uninstall the following kernel(s):" in out
+        assert (
+            "{} is the current running kernel.".format(current_kernel) in out
+        )
+
+    @pytest.mark.parametrize(
+        "remove,reinstall,expected_print,expected_prompt",
+        (
+            (
+                packages_to_remove,
+                packages_to_reinstall,
+                [
+                    mock.call(["remove1", "remove2"]),
+                    mock.call(["reinstall1", "reinstall2"]),
+                ],
+                [mock.call(messages.PROCEED_YES_NO)],
+            ),
+            (
+                packages_to_remove,
+                [],
+                [mock.call(["remove1", "remove2"])],
+                [mock.call(messages.PROCEED_YES_NO)],
+            ),
+            (
+                [],
+                packages_to_reinstall,
+                [mock.call(["reinstall1", "reinstall2"])],
+                [mock.call(messages.PROCEED_YES_NO)],
+            ),
+            ([], [], [], []),
+        ),
+    )
+    @mock.patch(M_PATH + "util.print_package_list")
+    @mock.patch(M_PATH + "util.prompt_for_confirmation")
+    def test_prompt_for_purge(
+        self,
+        m_confirmation,
+        m_print_packages,
+        remove,
+        reinstall,
+        expected_print,
+        expected_prompt,
+        entitlement_factory,
+    ):
+        m_confirmation.return_value = True
+        entitlement = entitlement_factory(
+            RepoTestEntitlement,
+            affordances={"series": ["xenial"]},
+            purge=True,
+        )
+
+        assert entitlement.prompt_for_purge(remove, reinstall) is True
+        assert m_print_packages.call_args_list == expected_print
+        assert m_confirmation.call_args_list == expected_prompt
+
+    @pytest.mark.parametrize(
+        "remove,expected_remove",
+        (
+            (
+                packages_to_remove,
+                [
+                    mock.call(
+                        ["remove1", "remove2"],
+                        messages.UNINSTALLING_PACKAGES_FAILED.format(
+                            packages=["remove1", "remove2"]
+                        ),
+                    ),
+                ],
+            ),
+            (
+                [],
+                [],
+            ),
+        ),
+    )
+    @mock.patch(M_PATH + "apt.purge_packages")
+    @mock.patch(M_PATH + "apt.get_installed_packages_names")
+    def test_execute_removal(
+        self,
+        m_installed_packages,
+        m_apt_purge,
+        remove,
+        expected_remove,
+        entitlement_factory,
+    ):
+        m_installed_packages.return_value = ["remove1", "remove2"]
+        entitlement = entitlement_factory(
+            RepoTestEntitlement,
+            affordances={"series": ["xenial"]},
+            purge=True,
+        )
+        entitlement.execute_removal(remove)
+
+        assert m_apt_purge.call_args_list == expected_remove
+
+    @pytest.mark.parametrize(
+        "reinstall,expected_install",
+        (
+            (
+                packages_to_reinstall,
+                [
+                    mock.call(
+                        ["reinstall1=2.0", "reinstall2=2.0"],
+                        apt_options=[
+                            "--allow-downgrades",
+                            '-o Dpkg::Options::="--force-confdef"',
+                            '-o Dpkg::Options::="--force-confold"',
+                        ],
+                        override_env_vars={
+                            "DEBIAN_FRONTEND": "noninteractive"
+                        },
+                    ),
+                ],
+            ),
+            ([], []),
+        ),
+    )
+    @mock.patch(M_PATH + "apt.run_apt_install_command")
+    @mock.patch(M_PATH + "apt.get_installed_packages_names")
+    def test_execute_reinstall(
+        self,
+        m_installed_packages,
+        m_apt_install,
+        reinstall,
+        expected_install,
+        entitlement_factory,
+    ):
+        m_installed_packages.return_value = ["reinstall1", "reinstall2"]
+        entitlement = entitlement_factory(
+            RepoTestEntitlement,
+            affordances={"series": ["xenial"]},
+            purge=True,
+        )
+        entitlement.execute_reinstall(reinstall)
+
+        assert m_apt_install.call_args_list == expected_install
+
+
 class TestRemoveAptConfig:
     def test_missing_aptURL(self, entitlement_factory):
         # Make aptURL missing
@@ -727,9 +1031,9 @@ class TestSetupAptConfig:
             (
                 RepoTestEntitlement,
                 {},
-                exceptions.UserFacingError,
+                exceptions.UbuntuProError,
                 "Ubuntu Pro server provided no aptKey directive for"
-                " repotest.",
+                " repotest",
             ),
             (
                 RepoTestEntitlement,
@@ -740,9 +1044,9 @@ class TestSetupAptConfig:
             (
                 RepoTestEntitlement,
                 {"aptKey": "somekey", "aptURL": "someURL"},
-                exceptions.UserFacingError,
-                "Empty repotest apt suites directive from"
-                " https://contracts.canonical.com",
+                exceptions.UbuntuProError,
+                "Ubuntu Pro server provided no suites directive for"
+                " repotest",
             ),
         ),
     )
@@ -859,13 +1163,14 @@ class TestSetupAptConfig:
         entitlement = entitlement_factory(
             RepoTestEntitlementRepoWithPin, affordances={"series": ["xenial"]}
         )
-        with pytest.raises(exceptions.UserFacingError) as excinfo:
+        with pytest.raises(exceptions.UbuntuProError) as excinfo:
             entitlement.setup_apt_config()
         assert (
-            "Cannot setup apt pin. Empty apt repo origin value 'None'."
+            "Cannot setup apt pin. Empty apt repo origin value for repotest"
             in str(excinfo.value)
         )
 
+    @mock.patch("uaclient.apt.update_sources_list")
     @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch(M_PATH + "apt.add_auth_apt_repo")
     @mock.patch(M_PATH + "apt.run_apt_update_command")
@@ -879,6 +1184,7 @@ class TestSetupAptConfig:
         m_add_ppa_pinning,
         m_run_apt_update_command,
         m_add_auth_repo,
+        m_update_sources_list,
         _m_setup_apt_proxy,
         entitlement_factory,
     ):
@@ -903,7 +1209,8 @@ class TestSetupAptConfig:
                 entitlement.repo_pin_priority,
             )
         ] == m_add_ppa_pinning.call_args_list
-        assert [mock.call()] == m_run_apt_update_command.call_args_list
+        assert [] == m_run_apt_update_command.call_args_list
+        assert 1 == m_update_sources_list.call_count
 
 
 class TestCheckAptURLIsApplied:
