@@ -1,23 +1,22 @@
-import textwrap
 from collections import defaultdict
 from datetime import datetime, timezone
 from enum import Enum
 from functools import lru_cache
 from random import choice
-from typing import Any, DefaultDict, Dict, List, Tuple, Union  # noqa: F401
+from typing import Any, DefaultDict, Dict, List, Tuple
 
-import apt  # type: ignore
+import apt_pkg  # type: ignore
 
-from uaclient import livepatch, messages
+from uaclient import exceptions, livepatch, messages
 from uaclient.api.u.pro.security.status.reboot_required.v1 import (
     _reboot_required,
 )
 from uaclient.api.u.pro.status.is_attached.v1 import _is_attached
 from uaclient.apt import (
     PreserveAptCfg,
-    get_apt_cache,
     get_apt_cache_datetime,
-    get_esm_cache,
+    get_apt_pkg_cache,
+    get_esm_apt_pkg_cache,
 )
 from uaclient.config import UAConfig
 from uaclient.entitlements import ESMAppsEntitlement, ESMInfraEntitlement
@@ -33,6 +32,7 @@ from uaclient.system import (
     is_current_series_lts,
     is_supported,
 )
+from uaclient.util import print_package_list
 
 ESM_SERVICES = ("esm-infra", "esm-apps")
 
@@ -58,23 +58,29 @@ def get_origin_information_to_service_map():
 
 
 def get_installed_packages_by_origin() -> DefaultDict[
-    "str", List[apt.package.Package]
+    "str", List[apt_pkg.Package]
 ]:
     result = defaultdict(list)
 
-    with PreserveAptCfg(get_apt_cache) as cache:
+    with PreserveAptCfg(get_apt_pkg_cache) as cache:
         installed_packages = [
-            package for package in cache if package.is_installed
+            package for package in cache.packages if package.current_ver
         ]
         result["all"] = installed_packages
 
+        dep_cache = apt_pkg.DepCache(cache)
+
         for package in installed_packages:
-            result[get_origin_for_package(package)].append(package)
+            result[
+                get_origin_for_installed_package(package, dep_cache)
+            ].append(package)
 
     return result
 
 
-def get_origin_for_package(package: apt.package.Package) -> str:
+def get_origin_for_installed_package(
+    package: apt_pkg.Package, dep_cache: apt_pkg.DepCache
+) -> str:
     """
     Returns the origin for a package installed in the system.
 
@@ -82,18 +88,22 @@ def get_origin_for_package(package: apt.package.Package) -> str:
     We check the available versions (installed, candidate) to determine the
     most reasonable origin for the package.
     """
-    available_origins = package.installed.origins
+    # We assume that packages we pass are installed
+    if not package.current_ver:
+        return ""
+    available_origins = package.current_ver.file_list
 
     # If the installed version for a package has a single origin, it means that
     # only the local dpkg reference is there. Then, we check if there is a
     # candidate version. No candidate means we don't know anything about the
     # package. Otherwise we check for the origins of the candidate version.
     if len(available_origins) == 1:
-        if package.candidate is None or package.installed == package.candidate:
+        candidate = dep_cache.get_candidate_ver(package)
+        if not candidate or package.current_ver == candidate:
             return "unknown"
-        available_origins = package.candidate.origins
+        available_origins = candidate.file_list
 
-    for origin in available_origins:
+    for origin, _ in available_origins:
         service = get_origin_information_to_service_map().get(
             (origin.origin, origin.archive), ""
         )
@@ -122,9 +132,9 @@ def get_update_status(service_name: str, ua_info: Dict[str, Any]) -> str:
     return UpdateStatus.UNAVAILABLE.value
 
 
-def filter_security_updates(
-    packages: List[apt.package.Package],
-) -> DefaultDict[str, List[Tuple[apt.package.Version, str]]]:
+def filter_updates(
+    packages: List[apt_pkg.Package],
+) -> DefaultDict[str, List[Tuple[apt_pkg.Version, str]]]:
     """Filters a list of packages looking for available updates.
 
     All versions greater than the installed one are reported, based on where
@@ -137,13 +147,15 @@ def filter_security_updates(
     # but has be advertised about esm packages. Since those
     # sources live in a private folder, we need a different apt cache
     # to access them.
-    with PreserveAptCfg(get_esm_cache) as esm_cache:
+    with PreserveAptCfg(get_esm_apt_pkg_cache) as esm_cache:
         for package in packages:
-            if package.is_installed:
-                for version in package.versions:
-                    if version > package.installed:
+            # We only care about installed packages here
+            if package.current_ver:
+                for version in package.version_list:
+                    # Seems mypy cannot understand we can compare these :/
+                    if version > package.current_ver:  # type: ignore
                         counted_as_security = False
-                        for origin in version.origins:
+                        for origin, _ in version.file_list:
                             service = (
                                 get_origin_information_to_service_map().get(
                                     (origin.origin, origin.archive)
@@ -155,7 +167,7 @@ def filter_security_updates(
                                 # No need to loop through all the origins
                                 break
                         # Also no need to report backports at least for now...
-                        expected_origin = version.origins[0]
+                        expected_origin = version.file_list[0][0]
                         if (
                             not counted_as_security
                             and "backports" not in expected_origin.archive
@@ -169,10 +181,10 @@ def filter_security_updates(
                 # previous one
                 if package.name in esm_cache:
                     esm_package = esm_cache[package.name]
-                    for version in esm_package.versions:
-                        if version > package.installed:
-                            for origin in version.origins:
-                                service = get_origin_information_to_service_map().get(  # noqa
+                    for version in esm_package.version_list:
+                        if version > package.current_ver:  # type: ignore
+                            for origin, _ in version.file_list:
+                                service = get_origin_information_to_service_map().get(  # noqa: E501
                                     (origin.origin, origin.archive)
                                 )
                                 if service:
@@ -218,7 +230,11 @@ def get_ua_info(cfg: UAConfig) -> Dict[str, Any]:
 
 # Yeah Any is bad, but so is python<3.8 without TypedDict
 def get_livepatch_fixed_cves() -> List[Dict[str, Any]]:
-    lp_status = livepatch.status()
+    try:
+        lp_status = livepatch.status()
+    except exceptions.ProcessExecutionError:
+        return []
+
     our_kernel_version = get_kernel_info().proc_version_signature_version
     if (
         lp_status is not None
@@ -238,19 +254,17 @@ def get_livepatch_fixed_cves() -> List[Dict[str, Any]]:
 
 
 def create_updates_list(
-    security_upgradable_versions: DefaultDict[
-        str, List[Tuple[apt.package.Version, str]]
-    ],
+    upgradable_versions: DefaultDict[str, List[Tuple[apt_pkg.Version, str]]],
     ua_info: Dict[str, Any],
 ) -> List[Dict[str, Any]]:
     updates = []
-    for service, version_list in security_upgradable_versions.items():
+    for service, version_list in upgradable_versions.items():
         status = get_update_status(service, ua_info)
         for version, origin in version_list:
             updates.append(
                 {
-                    "package": version.package.name,
-                    "version": version.version,
+                    "package": version.parent_pkg.name,
+                    "version": version.ver_str,
                     "service_name": service,
                     "status": status,
                     "origin": origin,
@@ -279,11 +293,11 @@ def security_status_dict(cfg: UAConfig) -> Dict[str, Any]:
     installed_packages = packages_by_origin["all"]
     summary["num_installed_packages"] = len(installed_packages)
 
-    security_upgradable_versions = filter_security_updates(installed_packages)
+    upgradable_versions = filter_updates(installed_packages)
     # This version of security-status only cares about security updates
-    security_upgradable_versions["standard-updates"] = []
+    upgradable_versions["standard-updates"] = []
 
-    updates = create_updates_list(security_upgradable_versions, ua_info)
+    updates = create_updates_list(upgradable_versions, ua_info)
 
     summary["num_main_packages"] = len(packages_by_origin["main"])
     summary["num_restricted_packages"] = len(packages_by_origin["restricted"])
@@ -296,14 +310,10 @@ def security_status_dict(cfg: UAConfig) -> Dict[str, Any]:
     summary["num_esm_infra_packages"] = len(packages_by_origin["esm-infra"])
     summary["num_esm_apps_packages"] = len(packages_by_origin["esm-apps"])
 
-    summary["num_esm_infra_updates"] = len(
-        security_upgradable_versions["esm-infra"]
-    )
-    summary["num_esm_apps_updates"] = len(
-        security_upgradable_versions["esm-apps"]
-    )
+    summary["num_esm_infra_updates"] = len(upgradable_versions["esm-infra"])
+    summary["num_esm_apps_updates"] = len(upgradable_versions["esm-apps"])
     summary["num_standard_security_updates"] = len(
-        security_upgradable_versions["standard-security"]
+        upgradable_versions["standard-security"]
     )
     summary["reboot_required"] = _reboot_required(cfg).reboot_required
 
@@ -316,7 +326,7 @@ def security_status_dict(cfg: UAConfig) -> Dict[str, Any]:
 
 
 def _print_package_summary(
-    package_lists: DefaultDict[str, List[apt.package.Package]],
+    package_lists: DefaultDict[str, List[apt_pkg.Package]],
     show_items: str = "all",
     always_show: bool = False,
 ) -> None:
@@ -332,10 +342,9 @@ def _print_package_summary(
             + len(package_lists["esm-infra"])
         )
         print(
-            messages.SS_SUMMARY_ARCHIVE.format(
+            messages.SS_SUMMARY_ARCHIVE.pluralize(packages_mr).format(
                 offset=offset,
                 count=packages_mr,
-                plural="s",
                 repository="Main/Restricted",
             )
         )
@@ -348,10 +357,9 @@ def _print_package_summary(
         )
         if packages_um or always_show:
             print(
-                messages.SS_SUMMARY_ARCHIVE.format(
+                messages.SS_SUMMARY_ARCHIVE.pluralize(packages_um).format(
                     offset=offset,
                     count=packages_um,
-                    plural="s" if packages_um > 1 else "",
                     repository="Universe/Multiverse",
                 )
             )
@@ -359,19 +367,21 @@ def _print_package_summary(
     if show_items in ("all", "third-party"):
         packages_thirdparty = len(package_lists["third-party"])
         if packages_thirdparty or always_show:
-            msg = messages.SS_SUMMARY_THIRD_PARTY_SN
-            if packages_thirdparty > 1:
-                msg = messages.SS_SUMMARY_THIRD_PARTY_PL
-            print(msg.format(offset=offset, count=packages_thirdparty))
+            print(
+                messages.SS_SUMMARY_THIRD_PARTY.pluralize(
+                    packages_thirdparty
+                ).format(offset=offset, count=packages_thirdparty)
+            )
 
     if show_items in ("all", "unknown"):
         packages_unknown = len(package_lists["unknown"])
         if packages_unknown or always_show:
             print(
-                messages.SS_SUMMARY_UNAVAILABLE.format(
+                messages.SS_SUMMARY_UNAVAILABLE.pluralize(
+                    packages_unknown
+                ).format(
                     offset=offset,
                     count=packages_unknown,
-                    plural="s" if packages_unknown > 1 else "",
                 )
             )
 
@@ -420,16 +430,17 @@ def _print_service_support(
         )
 
     if installed_updates:
-        message += messages.SS_SERVICE_ENABLED_COUNTS.format(
+        message += " " + messages.SS_SERVICE_ENABLED_COUNTS.pluralize(
+            installed_updates
+        ).format(
             updates=installed_updates,
-            plural="" if installed_updates == 1 else "s",
         )
 
     if available_updates:
-        message += messages.SS_SERVICE_ADVERTISE_COUNTS.format(
-            verb="is" if available_updates == 1 else "are",
+        message += " " + messages.SS_SERVICE_ADVERTISE_COUNTS.pluralize(
+            available_updates
+        ).format(
             updates=available_updates,
-            plural="s" if available_updates > 1 else "",
         )
     print(message)
 
@@ -444,23 +455,6 @@ def _print_service_support(
     print("")
 
 
-def _print_package_list(
-    package_list: List[str],
-):
-    package_names = " ".join(package_list)
-    print(
-        "\n".join(
-            textwrap.wrap(
-                package_names,
-                width=80,
-                break_long_words=False,
-                break_on_hyphens=False,
-            )
-        )
-    )
-    print("")
-
-
 def _print_apt_update_call():
     last_apt_update = get_apt_cache_datetime()
     if last_apt_update is None:
@@ -488,13 +482,13 @@ def security_status(cfg: UAConfig):
     is_attached = get_ua_info(cfg)["attached"]
 
     packages_by_origin = get_installed_packages_by_origin()
-    security_upgradable_versions_infra = filter_security_updates(
+    security_upgradable_versions_infra = filter_updates(
         packages_by_origin["main"]
         + packages_by_origin["restricted"]
         + packages_by_origin["esm-infra"],
     )["esm-infra"]
 
-    security_upgradable_versions_apps = filter_security_updates(
+    security_upgradable_versions_apps = filter_updates(
         packages_by_origin["universe"]
         + packages_by_origin["multiverse"]
         + packages_by_origin["esm-apps"],
@@ -516,10 +510,10 @@ def security_status(cfg: UAConfig):
     if esm_infra_status == ApplicationStatus.DISABLED:
         _print_lts_support()
 
-    not_attached = ""
-    if not is_attached:
-        not_attached = " NOT"
-    print(messages.SS_IS_ATTACHED.format(not_attached=not_attached))
+    if is_attached:
+        print(messages.SS_IS_ATTACHED)
+    else:
+        print(messages.SS_IS_NOT_ATTACHED)
     print("")
 
     _print_service_support(
@@ -564,8 +558,8 @@ def list_third_party_packages():
         print(messages.SS_THIRD_PARTY)
 
         print("")
-        print("Packages:")
-        _print_package_list(package_names)
+        print(messages.SS_PACKAGES_HEADER)
+        print_package_list(package_names)
         print(messages.SS_SHOW_HINT.format(package=choice(package_names)))
     else:
         print(messages.SS_NO_THIRD_PARTY)
@@ -584,8 +578,8 @@ def list_unavailable_packages():
         print(messages.SS_UNAVAILABLE)
         print("")
 
-        print("Packages:")
-        _print_package_list(package_names)
+        print(messages.SS_PACKAGES_HEADER)
+        print_package_list(package_names)
         print(messages.SS_SHOW_HINT.format(package=choice(package_names)))
 
     else:
@@ -600,11 +594,11 @@ def list_esm_infra_packages(cfg):
     all_infra_packages = infra_packages + mr_packages
 
     infra_updates = set()
-    security_upgradable_versions = filter_security_updates(all_infra_packages)[
+    security_upgradable_versions = filter_updates(all_infra_packages)[
         "esm-infra"
     ]
     for update, _ in security_upgradable_versions:
-        infra_updates.add(update.package)
+        infra_updates.add(update.parent_pkg)
 
     series = get_release_info().series
     is_lts = is_current_series_lts()
@@ -658,22 +652,23 @@ def list_esm_infra_packages(cfg):
     if not is_supported(series):
         if available_package_names:
             print(messages.SS_UPDATES_AVAILABLE.format(service="esm-infra"))
-            _print_package_list(available_package_names)
+            print_package_list(available_package_names)
 
         if installed_package_names:
             print(messages.SS_UPDATES_INSTALLED.format(service="esm-infra"))
-            _print_package_list(installed_package_names)
+            print_package_list(installed_package_names)
 
         hint_list = available_package_names or installed_package_names
         # Check names because packages may have been already listed
         if remaining_package_names:
-            print(
-                messages.SS_OTHER_PACKAGES.format(
-                    prefix="Further installed" if hint_list else "Installed",
-                    service="esm-infra",
+            if hint_list:
+                msg = messages.SS_FURTHER_OTHER_PACKAGES.format(
+                    service="esm-infra"
                 )
-            )
-            _print_package_list(remaining_package_names)
+            else:
+                msg = messages.SS_OTHER_PACKAGES.format(service="esm-infra")
+            print(msg)
+            print_package_list(remaining_package_names)
 
         if hint_list:
             print(messages.SS_SHOW_HINT.format(package=choice(hint_list)))
@@ -689,11 +684,11 @@ def list_esm_apps_packages(cfg):
     all_apps_packages = apps_packages + um_packages
 
     apps_updates = set()
-    security_upgradable_versions = filter_security_updates(all_apps_packages)[
+    security_upgradable_versions = filter_updates(all_apps_packages)[
         "esm-apps"
     ]
     for update, _ in security_upgradable_versions:
-        apps_updates.add(update.package)
+        apps_updates.add(update.parent_pkg)
 
     is_lts = is_current_series_lts()
 
@@ -738,23 +733,24 @@ def list_esm_apps_packages(cfg):
     if all_apps_packages:
         if available_package_names:
             print(messages.SS_UPDATES_AVAILABLE.format(service="esm-apps"))
-            _print_package_list(available_package_names)
+            print_package_list(available_package_names)
 
         if installed_package_names:
             print(messages.SS_UPDATES_INSTALLED.format(service="esm-apps"))
-            _print_package_list(installed_package_names)
+            print_package_list(installed_package_names)
 
         hint_list = available_package_names or installed_package_names
 
         # Check names because packages may have been already listed
         if remaining_package_names:
-            print(
-                messages.SS_OTHER_PACKAGES.format(
-                    prefix="Further installed" if hint_list else "Installed",
-                    service="esm-apps",
+            if hint_list:
+                msg = messages.SS_FURTHER_OTHER_PACKAGES.format(
+                    service="esm-apps"
                 )
-            )
-            _print_package_list(remaining_package_names)
+            else:
+                msg = messages.SS_OTHER_PACKAGES.format(service="esm-apps")
+            print(msg)
+            print_package_list(remaining_package_names)
 
         if hint_list:
             print(messages.SS_SHOW_HINT.format(package=choice(hint_list)))
