@@ -3,25 +3,17 @@ import json
 import mock
 import pytest
 
-from uaclient import exceptions, messages
-from uaclient.actions import (
-    attach_with_token,
-    auto_attach,
-    collect_logs,
-    get_cloud_instance,
-)
-from uaclient.exceptions import (
-    CloudFactoryError,
-    CloudFactoryNoCloudError,
-    CloudFactoryNonViableCloudError,
-    CloudFactoryUnsupportedCloudError,
-    ContractAPIError,
-    NonAutoAttachImageError,
-    UserFacingError,
-)
-from uaclient.testing import helpers
+from uaclient import exceptions
+from uaclient.actions import attach_with_token, auto_attach, collect_logs
+from uaclient.testing import fakes, helpers
 
 M_PATH = "uaclient.actions."
+APPARMOR_DENIED = (
+    'audit: type=1400 audit(1703513431.601:36): apparmor="DENIED" '
+    'operation="open" profile="ubuntu_pro_apt_news" '
+    'name="/proc/1422/status" pid=1422 comm="python3" '
+    'requested_mask="r" denied_mask="r" fsuid=0 ouid=0'
+)
 
 
 def fake_instance_factory():
@@ -57,7 +49,7 @@ class TestAttachWithToken:
             (
                 "token",
                 True,
-                exceptions.UrlError(Exception(), "url"),
+                exceptions.ConnectivityError(cause=Exception(), url="url"),
                 None,
                 None,
                 None,
@@ -81,7 +73,7 @@ class TestAttachWithToken:
                 [{"machineTokenInfo": {"machineId": "machine-id"}}],
                 "get-machine-id-result",
                 mock.sentinel.entitlements,
-                exceptions.UrlError(Exception(), "url"),
+                exceptions.ConnectivityError(cause=Exception(), url="url"),
                 None,
                 [mock.call(contract_token="token", attachment_dt=mock.ANY)],
                 [mock.call({"machineTokenInfo": {"machineId": "machine-id"}})],
@@ -94,7 +86,7 @@ class TestAttachWithToken:
                 [mock.call()],
                 [],
                 [],
-                pytest.raises(exceptions.UrlError),
+                pytest.raises(exceptions.ConnectivityError),
             ),
             (
                 "token",
@@ -102,7 +94,7 @@ class TestAttachWithToken:
                 [{"machineTokenInfo": {"machineId": "machine-id"}}],
                 "get-machine-id-result",
                 mock.sentinel.entitlements,
-                exceptions.UserFacingError("message"),
+                fakes.FakeUbuntuProError(),
                 None,
                 [mock.call(contract_token="token", attachment_dt=mock.ANY)],
                 [mock.call({"machineTokenInfo": {"machineId": "machine-id"}})],
@@ -115,7 +107,7 @@ class TestAttachWithToken:
                 [mock.call()],
                 [],
                 [],
-                pytest.raises(exceptions.UserFacingError),
+                pytest.raises(exceptions.UbuntuProError),
             ),
             (
                 "token",
@@ -327,12 +319,14 @@ class TestAutoAttach:
         """Any unexpected errors will be raised."""
         cfg = FakeConfig()
 
-        unexpected_error = ContractAPIError(
-            "http://me", 500, json.dumps({"message": "something unexpected"})
+        unexpected_error = exceptions.ContractAPIError(
+            url="http://me",
+            code=500,
+            body=json.dumps({"message": "something unexpected"}),
         )
         m_get_contract_token_for_cloud_instances.side_effect = unexpected_error
 
-        with pytest.raises(ContractAPIError) as excinfo:
+        with pytest.raises(exceptions.ContractAPIError) as excinfo:
             auto_attach(cfg, fake_instance_factory())
 
         assert unexpected_error == excinfo.value
@@ -347,8 +341,10 @@ class TestCollectLogs:
     @mock.patch("uaclient.actions._get_state_files")
     @mock.patch("glob.glob")
     @mock.patch("uaclient.log.get_user_log_file")
+    @mock.patch("uaclient.system.subp", return_value=(APPARMOR_DENIED, ""))
     def test_collect_logs_invalid_file(
         self,
+        m_system_subp,
         m_get_user,
         m_glob,
         m_get_state_files,
@@ -374,65 +370,20 @@ class TestCollectLogs:
             mock.call("a"),
             mock.call("b"),
         ] == m_load_file.call_args_list
-        assert 2 == m_write_file.call_count
+        assert 3 == m_write_file.call_count
+
+        # apparmor checks
+        assert 1 == m_system_subp.call_count
+        assert [
+            mock.call(["journalctl", "-b", "-k", "--since=1 day ago"]),
+        ] == m_system_subp.call_args_list
+
         print(m_write_file.call_args_list)
         assert [
             mock.call("test/user0.log", "test"),
             mock.call("test/b", "test"),
+            mock.call("test/apparmor_logs.txt", APPARMOR_DENIED),
         ] == m_write_file.call_args_list
         assert [
             mock.call("Failed to load file: %s\n%s", "a", "test")
         ] in m_log_warning.call_args_list
-
-
-class TestGetCloudInstance:
-    @pytest.mark.parametrize(
-        "cloud_factory_error, expected_error_cls, expected_error_msg",
-        [
-            (
-                CloudFactoryNoCloudError("test"),
-                UserFacingError,
-                messages.UNABLE_TO_DETERMINE_CLOUD_TYPE,
-            ),
-            (
-                CloudFactoryNonViableCloudError("test"),
-                UserFacingError,
-                messages.UNSUPPORTED_AUTO_ATTACH,
-            ),
-            (
-                CloudFactoryUnsupportedCloudError("test"),
-                NonAutoAttachImageError,
-                messages.UNSUPPORTED_AUTO_ATTACH_CLOUD_TYPE.format(
-                    cloud_type="test"
-                ),
-            ),
-            (
-                CloudFactoryNoCloudError("test"),
-                UserFacingError,
-                messages.UNABLE_TO_DETERMINE_CLOUD_TYPE,
-            ),
-            (
-                CloudFactoryError("test"),
-                UserFacingError,
-                messages.UNABLE_TO_DETERMINE_CLOUD_TYPE,
-            ),
-        ],
-    )
-    @mock.patch(M_PATH + "identity.cloud_instance_factory")
-    def test_handle_cloud_factory_errors(
-        self,
-        m_cloud_instance_factory,
-        cloud_factory_error,
-        expected_error_cls,
-        expected_error_msg,
-        FakeConfig,
-    ):
-        """Non-supported clouds will error."""
-        m_cloud_instance_factory.side_effect = cloud_factory_error
-        cfg = FakeConfig()
-
-        with pytest.raises(expected_error_cls) as excinfo:
-            get_cloud_instance(cfg=cfg)
-
-        if expected_error_msg:
-            assert expected_error_msg == str(excinfo.value)
