@@ -5,6 +5,7 @@ import os
 import stat
 import subprocess
 from textwrap import dedent
+from typing import List, Optional
 
 import mock
 import pytest
@@ -17,21 +18,25 @@ from uaclient.apt import (
     APT_HELPER_TIMEOUT,
     APT_KEYS_DIR,
     APT_PROXY_CONF_FILE,
+    APT_PROXY_CONFIG_HEADER,
     APT_RETRIES,
     KEYRINGS_DIR,
+    SERIES_NOT_USING_DEB822,
     PreserveAptCfg,
+    _ensure_esm_cache_structure,
     add_apt_auth_conf_entry,
     add_auth_apt_repo,
     add_ppa_pinning,
     assert_valid_apt_credentials,
-    clean_apt_files,
-    compare_versions,
     find_apt_list_files,
     get_apt_cache_policy,
     get_apt_cache_time,
     get_apt_config_values,
+    get_installed_packages_by_origin,
     get_installed_packages_names,
     get_pkg_candidate_version,
+    get_remote_versions_for_package,
+    get_system_sources_file,
     is_installed,
     remove_apt_list_files,
     remove_auth_apt_repo,
@@ -39,11 +44,12 @@ from uaclient.apt import (
     run_apt_update_command,
     setup_apt_proxy,
     update_esm_caches,
+    update_sources_list,
 )
 from uaclient.entitlements.base import UAEntitlement
 from uaclient.entitlements.entitlement_status import ApplicationStatus
 from uaclient.entitlements.repo import RepoEntitlement
-from uaclient.entitlements.tests.test_repo import RepoTestEntitlement
+from uaclient.testing import fakes
 
 POST_INSTALL_APT_CACHE_NO_UPDATES = """
 -32768 https://esm.ubuntu.com/ubuntu/ {0}-updates/main amd64 Packages
@@ -58,6 +64,52 @@ b/release-updates, now 1.2+3 arch123 [i,a]
 """
 
 
+def mock_origin(
+    component: str, archive: str, origin: str, site: str
+) -> mock.MagicMock:
+    mock_origin = mock.MagicMock()
+    mock_origin.component = component
+    mock_origin.archive = archive
+    mock_origin.origin = origin
+    mock_origin.site = site
+    return [mock_origin, 0]
+
+
+def mock_version(
+    version: str,
+    origin_list: List[mock.MagicMock] = [],
+    size: int = 1,
+) -> mock.MagicMock:
+    mock_version = mock.MagicMock()
+    mock_version.__gt__ = lambda self, other: self.ver_str > other.ver_str
+    mock_version.ver_str = version
+    mock_version.file_list = origin_list
+    mock_version.size = size
+    return mock_version
+
+
+def mock_package(
+    name: str,
+    installed_version: Optional[mock.MagicMock] = None,
+    other_versions: List[mock.MagicMock] = [],
+):
+    mock_package = mock.MagicMock()
+    mock_package.name = name
+    mock_package.version_list = []
+
+    mock_package.current_ver = None
+    if installed_version:
+        mock_package.current_ver = installed_version
+        installed_version.parent_pkg = mock_package
+        mock_package.version_list.append(installed_version)
+
+    for version in other_versions:
+        version.parent_pkg = mock_package
+        mock_package.version_list.append(version)
+
+    return mock_package
+
+
 class TestAddPPAPinning:
     @mock.patch("uaclient.system.get_release_info")
     def test_write_apt_pin_file_to_apt_preferences(
@@ -181,7 +233,12 @@ class TestValidAptCredentials:
             (
                 1,
                 "something broke",
-                "Unexpected APT error. See /var/log/ubuntu-advantage.log",
+                (
+                    "Unexpected APT error.\n"
+                    "Failed running command 'apt-helper ' [exit(1)]. Message: "
+                    "something broke\n"
+                    "See /var/log/ubuntu-advantage.log"
+                ),
             ),
             (
                 100,
@@ -224,7 +281,7 @@ class TestValidAptCredentials:
             stderr=stderr,
         )
 
-        with pytest.raises(exceptions.UserFacingError) as excinfo:
+        with pytest.raises(exceptions.UbuntuProError) as excinfo:
             assert_valid_apt_credentials(
                 repo_url="http://fakerepo", username="user", password="pwd"
             )
@@ -261,7 +318,7 @@ class TestValidAptCredentials:
         m_subp.side_effect = subprocess.TimeoutExpired(
             "something timed out", timeout=1000000
         )
-        with pytest.raises(exceptions.UserFacingError) as excinfo:
+        with pytest.raises(exceptions.UbuntuProError) as excinfo:
             assert_valid_apt_credentials(
                 repo_url="http://fakerepo", username="user", password="pwd"
             )
@@ -292,14 +349,12 @@ class TestValidAptCredentials:
 
 
 class TestAddAuthAptRepo:
+    @pytest.mark.parametrize("series", ("xenial", "noble"))
     @mock.patch("uaclient.apt.gpg.export_gpg_key")
     @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.apt.get_apt_auth_file_from_apt_config")
     @mock.patch("uaclient.apt.assert_valid_apt_credentials")
-    @mock.patch(
-        "uaclient.system.get_release_info",
-        return_value=mock.MagicMock(series="xenial"),
-    )
+    @mock.patch("uaclient.system.get_release_info")
     def test_add_auth_apt_repo_writes_sources_file(
         self,
         m_get_release_info,
@@ -307,6 +362,7 @@ class TestAddAuthAptRepo:
         m_get_apt_auth_file,
         m_subp,
         m_gpg_export,
+        series,
         tmpdir,
     ):
         """Write a properly configured sources file to repo_filename."""
@@ -314,33 +370,44 @@ class TestAddAuthAptRepo:
         auth_file = tmpdir.join("auth.conf").strpath
         m_get_apt_auth_file.return_value = auth_file
         m_subp.return_value = "500 esm.canonical.com...", ""  # apt policy
+        m_get_release_info.return_value = mock.MagicMock(series=series)
 
         add_auth_apt_repo(
             repo_filename=repo_file,
             repo_url="http://fakerepo",
             credentials="mycreds",
-            suites=("xenial",),
+            suites=(series,),
             keyring_file="keyring",
         )
 
-        expected_content = (
-            "deb http://fakerepo xenial main\n"
-            "# deb-src http://fakerepo xenial main\n"
-        )
+        if series in SERIES_NOT_USING_DEB822:
+            expected_content = (
+                "deb http://fakerepo {series} main\n"
+                "# deb-src http://fakerepo {series} main\n"
+            ).format(series=series)
+            src_keyfile = os.path.join(KEYRINGS_DIR, "keyring")
+            dest_keyfile = os.path.join(APT_KEYS_DIR, "keyring")
+            gpg_export_calls = [mock.call(src_keyfile, dest_keyfile)]
+        else:
+            expected_content = (
+                "# Written by ubuntu-pro-client\n"
+                "Types: deb\n"
+                "URIs: http://fakerepo\n"
+                "Suites: {series}\n"
+                "Components: main\n"
+                "Signed-By: /usr/share/keyrings/keyring\n"
+            ).format(series=series)
+            gpg_export_calls = []
+
         assert expected_content == system.load_file(repo_file)
-        src_keyfile = os.path.join(KEYRINGS_DIR, "keyring")
-        dest_keyfile = os.path.join(APT_KEYS_DIR, "keyring")
-        gpg_export_calls = [mock.call(src_keyfile, dest_keyfile)]
         assert gpg_export_calls == m_gpg_export.call_args_list
 
+    @pytest.mark.parametrize("series", ("xenial", "noble"))
     @mock.patch("uaclient.apt.gpg.export_gpg_key")
     @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.apt.get_apt_auth_file_from_apt_config")
     @mock.patch("uaclient.apt.assert_valid_apt_credentials")
-    @mock.patch(
-        "uaclient.system.get_release_info",
-        return_value=mock.MagicMock(series="xenial"),
-    )
+    @mock.patch("uaclient.system.get_release_info")
     def test_add_auth_apt_repo_ignores_suites_not_matching_series(
         self,
         m_get_release_info,
@@ -348,48 +415,67 @@ class TestAddAuthAptRepo:
         m_get_apt_auth_file,
         m_subp,
         m_gpg_export,
+        series,
         tmpdir,
     ):
         """Skip any apt suites that don't match the current series."""
         repo_file = tmpdir.join("repo.conf").strpath
         auth_file = tmpdir.join("auth.conf").strpath
         m_get_apt_auth_file.return_value = auth_file
-        # apt policy with xenial-updates enabled
+        # apt policy with series-updates enabled
         stdout = dedent(
             """\
-            500 http://archive.ubuntu.com/ xenial-updates/main amd64 \
+            500 http://archive.ubuntu.com/ {series}-updates/main amd64 \
                         Packages
-                release v=16.04,o=Ubuntu,a=xenial-updates,n=xenial,l=Ubuntu\
-                        ,c=main"""
+                release v=XX.04,o=Ubuntu,a={series}-updates,n={series},\
+                        l=Ubuntu,c=main""".format(
+                series=series
+            )
         )
         m_subp.return_value = stdout, ""
+        m_get_release_info.return_value = mock.MagicMock(series=series)
 
         add_auth_apt_repo(
             repo_filename=repo_file,
             repo_url="http://fakerepo",
             credentials="mycreds",
-            suites=("xenial-one", "xenial-updates", "trusty-gone"),
+            suites=(
+                "{}-one".format(series),
+                "{}-updates".format(series),
+                "trusty-gone",
+            ),
             keyring_file="keyring",
         )
 
-        expected_content = dedent(
-            """\
-            deb http://fakerepo xenial-one main
-            # deb-src http://fakerepo xenial-one main
-            deb http://fakerepo xenial-updates main
-            # deb-src http://fakerepo xenial-updates main
-        """
-        )
+        if series in SERIES_NOT_USING_DEB822:
+            expected_content = dedent(
+                """\
+                deb http://fakerepo {series}-one main
+                # deb-src http://fakerepo {series}-one main
+                deb http://fakerepo {series}-updates main
+                # deb-src http://fakerepo {series}-updates main
+            """
+            ).format(series=series)
+        else:
+            expected_content = dedent(
+                """\
+                # Written by ubuntu-pro-client
+                Types: deb
+                URIs: http://fakerepo
+                Suites: {series}-one {series}-updates
+                Components: main
+                Signed-By: /usr/share/keyrings/keyring
+            """
+            ).format(series=series)
+
         assert expected_content == system.load_file(repo_file)
 
+    @pytest.mark.parametrize("series", ("xenial", "noble"))
     @mock.patch("uaclient.apt.gpg.export_gpg_key")
     @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.apt.get_apt_auth_file_from_apt_config")
     @mock.patch("uaclient.apt.assert_valid_apt_credentials")
-    @mock.patch(
-        "uaclient.system.get_release_info",
-        return_value=mock.MagicMock(series="xenial"),
-    )
+    @mock.patch("uaclient.system.get_release_info")
     def test_add_auth_apt_repo_comments_updates_suites_on_non_update_machine(
         self,
         m_get_release_info,
@@ -397,35 +483,53 @@ class TestAddAuthAptRepo:
         m_get_apt_auth_file,
         m_subp,
         m_gpg_export,
+        series,
         tmpdir,
     ):
         """Skip any apt suites that don't match the current series."""
         repo_file = tmpdir.join("repo.conf").strpath
         auth_file = tmpdir.join("auth.conf").strpath
         m_get_apt_auth_file.return_value = auth_file
-        # apt policy without xenial-updates enabled
-        origin = "test-origin"
+        # apt policy without series-updates enabled
         m_subp.return_value = (
-            POST_INSTALL_APT_CACHE_NO_UPDATES.format("xenial", origin),
+            POST_INSTALL_APT_CACHE_NO_UPDATES.format(series, "test-origin"),
             "",
         )
+        m_get_release_info.return_value = mock.MagicMock(series=series)
 
         add_auth_apt_repo(
             repo_filename=repo_file,
             repo_url="http://fakerepo",
             credentials="mycreds",
-            suites=("xenial-one", "xenial-updates", "trusty-gone"),
+            suites=(
+                "{}-one".format(series),
+                "{}-updates".format(series),
+                "trusty-gone",
+            ),
             keyring_file="keyring",
         )
 
-        expected_content = dedent(
-            """\
-            deb http://fakerepo xenial-one main
-            # deb-src http://fakerepo xenial-one main
-            # deb http://fakerepo xenial-updates main
-            # deb-src http://fakerepo xenial-updates main
-        """
-        )
+        if series in SERIES_NOT_USING_DEB822:
+            expected_content = dedent(
+                """\
+                deb http://fakerepo {series}-one main
+                # deb-src http://fakerepo {series}-one main
+                # deb http://fakerepo {series}-updates main
+                # deb-src http://fakerepo {series}-updates main
+            """
+            ).format(series=series)
+        else:
+            expected_content = dedent(
+                """\
+                # Written by ubuntu-pro-client
+                Types: deb
+                URIs: http://fakerepo
+                Suites: {series}-one
+                Components: main
+                Signed-By: /usr/share/keyrings/keyring
+            """
+            ).format(series=series)
+
         assert expected_content == system.load_file(repo_file)
 
     @mock.patch("uaclient.apt.gpg.export_gpg_key")
@@ -584,7 +688,7 @@ class TestCleanAptFiles:
 
         class TestRepo(request.param):
             name = entitlement_name
-            repo_list_file_tmpl = repo_tmpl
+            repo_file_tmpl = repo_tmpl
             repo_pref_file_tmpl = pref_tmpl
             is_repo = request.param == RepoEntitlement
 
@@ -600,51 +704,6 @@ class TestCleanAptFiles:
 
         return TestRepo
 
-    @mock.patch("os.path.exists", return_value=True)
-    @mock.patch("uaclient.system.ensure_file_absent")
-    def test_removals_for_repo_entitlements(
-        self, m_ensure_file_absent, _m_path_exists
-    ):
-        m_entitlements = mock.Mock()
-        m_entitlements.ENTITLEMENT_CLASSES = [RepoTestEntitlement]
-
-        clean_apt_files(_entitlements=m_entitlements)
-
-        assert 2 == m_ensure_file_absent.call_count
-
-    def test_files_for_all_series_removed(self, mock_apt_entitlement, tmpdir):
-        m_entitlements = mock.Mock()
-        m_entitlements.ENTITLEMENT_CLASSES = [mock_apt_entitlement]
-
-        clean_apt_files(_entitlements=m_entitlements)
-
-        if mock_apt_entitlement.is_repo:
-            assert [] == tmpdir.listdir()
-        else:
-            assert sorted(
-                [tmpdir.join("source-test_ent"), tmpdir.join("pref-test_ent")]
-            ) == sorted(tmpdir.listdir())
-
-    def test_other_files_not_removed(self, mock_apt_entitlement, tmpdir):
-        other_filename = "other_file-acidic"
-        tmpdir.join(other_filename).ensure()
-
-        m_entitlements = mock.Mock()
-        m_entitlements.ENTITLEMENT_CLASSES = [mock_apt_entitlement]
-
-        clean_apt_files(_entitlements=m_entitlements)
-
-        if mock_apt_entitlement.is_repo:
-            assert [tmpdir.join(other_filename)] == tmpdir.listdir()
-        else:
-            assert sorted(
-                [
-                    tmpdir.join("source-test_ent"),
-                    tmpdir.join("pref-test_ent"),
-                    tmpdir.join(other_filename),
-                ]
-            ) == sorted(tmpdir.listdir())
-
 
 @pytest.fixture(params=(mock.sentinel.default, None, "some_string"))
 def remove_auth_apt_repo_kwargs(request):
@@ -661,15 +720,20 @@ def remove_auth_apt_repo_kwargs(request):
     return kwargs
 
 
+@mock.patch("uaclient.apt.system.subp")
+@mock.patch("uaclient.apt.remove_repo_from_apt_auth_file")
+@mock.patch("uaclient.apt.system.ensure_file_absent")
 class TestRemoveAuthAptRepo:
-    @mock.patch("uaclient.apt.system.subp")
-    @mock.patch("uaclient.apt.remove_repo_from_apt_auth_file")
-    @mock.patch("uaclient.apt.system.ensure_file_absent")
     def test_repo_file_deleted(
-        self, m_ensure_file_absent, _mock, __mock, remove_auth_apt_repo_kwargs
+        self,
+        m_ensure_file_absent,
+        _m_remove_repo,
+        _m_subp,
+        remove_auth_apt_repo_kwargs,
     ):
         """Ensure that repo_filename is deleted, regardless of other params."""
-        repo_filename, repo_url = mock.sentinel.filename, mock.sentinel.url
+        repo_filename = "/etc/apt/sources.list.d/pro-repofile.list"
+        repo_url = mock.sentinel.url
 
         remove_auth_apt_repo(
             repo_filename, repo_url, **remove_auth_apt_repo_kwargs
@@ -677,14 +741,36 @@ class TestRemoveAuthAptRepo:
 
         assert mock.call(repo_filename) in m_ensure_file_absent.call_args_list
 
-    @mock.patch("uaclient.apt.system.subp")
-    @mock.patch("uaclient.apt.system.ensure_file_absent")
-    @mock.patch("uaclient.apt.remove_repo_from_apt_auth_file")
+    def test_old_repo_file_deleted_when_deb822(
+        self,
+        m_ensure_file_absent,
+        _m_remove_repo,
+        _m_subp,
+        remove_auth_apt_repo_kwargs,
+    ):
+        repo_filename = "/etc/apt/sources.list.d/pro-repofile.sources"
+        repo_url = mock.sentinel.url
+
+        remove_auth_apt_repo(
+            repo_filename, repo_url, **remove_auth_apt_repo_kwargs
+        )
+
+        assert mock.call(repo_filename) in m_ensure_file_absent.call_args_list
+        assert (
+            mock.call("/etc/apt/sources.list.d/pro-repofile.list")
+            in m_ensure_file_absent.call_args_list
+        )
+
     def test_remove_from_auth_file_called(
-        self, m_remove_repo, _mock, __mock, remove_auth_apt_repo_kwargs
+        self,
+        _m_ensure_file_absent,
+        m_remove_repo,
+        _m_subp,
+        remove_auth_apt_repo_kwargs,
     ):
         """Ensure that remove_repo_from_apt_auth_file is called."""
-        repo_filename, repo_url = mock.sentinel.filename, mock.sentinel.url
+        repo_filename = "/etc/apt/sources.list.d/pro-repofile.list"
+        repo_url = mock.sentinel.url
 
         remove_auth_apt_repo(
             repo_filename, repo_url, **remove_auth_apt_repo_kwargs
@@ -692,14 +778,16 @@ class TestRemoveAuthAptRepo:
 
         assert mock.call(repo_url) in m_remove_repo.call_args_list
 
-    @mock.patch("uaclient.apt.system.subp")
-    @mock.patch("uaclient.apt.remove_repo_from_apt_auth_file")
-    @mock.patch("uaclient.apt.system.ensure_file_absent")
     def test_keyring_file_deleted_if_given(
-        self, m_ensure_file_absent, _mock, __mock, remove_auth_apt_repo_kwargs
+        self,
+        m_ensure_file_absent,
+        _m_remove_repo,
+        _m_subp,
+        remove_auth_apt_repo_kwargs,
     ):
         """We should always delete the keyring file if it is given"""
-        repo_filename, repo_url = mock.sentinel.filename, mock.sentinel.url
+        repo_filename = "/etc/apt/sources.list.d/pro-repofile.list"
+        repo_url = mock.sentinel.url
 
         remove_auth_apt_repo(
             repo_filename, repo_url, **remove_auth_apt_repo_kwargs
@@ -865,7 +953,7 @@ class TestRunAptCommand:
                     "APT update failed.",
                     (
                         "APT update failed to read APT config "
-                        "for the following URLs:"
+                        "for the following:"
                     ),
                     "- t1",
                     "- t2",
@@ -877,7 +965,7 @@ class TestRunAptCommand:
                     "APT update failed.",
                     (
                         "APT update failed to read APT config "
-                        "for the following URL:"
+                        "for the following:"
                     ),
                     "- t1",
                 ),
@@ -891,7 +979,7 @@ class TestRunAptCommand:
                     "APT update failed.",
                     (
                         "APT update failed to read APT config "
-                        "for the following URL:"
+                        "for the following:"
                     ),
                     "- t1",
                 ),
@@ -908,10 +996,10 @@ class TestRunAptCommand:
             cmd="apt update", stderr=error_msg
         )
 
-        with pytest.raises(exceptions.UserFacingError) as excinfo:
+        with pytest.raises(exceptions.UbuntuProError) as excinfo:
             run_apt_update_command()
 
-        expected_message = "\n".join(output_list) + "."
+        expected_message = "\n".join(output_list)
         assert expected_message == excinfo.value.msg
 
     @mock.patch("uaclient.apt.system.subp")
@@ -938,7 +1026,7 @@ class TestRunAptCommand:
     ):
         m_subp.side_effect = [
             ("policy1", ""),
-            exceptions.UserFacingError("test"),
+            fakes.FakeUbuntuProError(),
             ("policy2", ""),
         ]
 
@@ -946,7 +1034,7 @@ class TestRunAptCommand:
         # Confirming that caching is happening
         assert "policy1" == get_apt_cache_policy()
 
-        with pytest.raises(exceptions.UserFacingError):
+        with pytest.raises(exceptions.UbuntuProError):
             run_apt_update_command()
 
         # Confirm cache was cleared
@@ -965,7 +1053,7 @@ class TestAptProxyConfig:
                 [
                     mock.call(
                         APT_PROXY_CONF_FILE,
-                        messages.APT_PROXY_CONFIG_HEADER
+                        APT_PROXY_CONFIG_HEADER
                         + APT_CONFIG_GLOBAL_PROXY_HTTP.format(
                             proxy_url="mock_http_proxy"
                         ),
@@ -979,7 +1067,7 @@ class TestAptProxyConfig:
                 [
                     mock.call(
                         APT_PROXY_CONF_FILE,
-                        messages.APT_PROXY_CONFIG_HEADER
+                        APT_PROXY_CONFIG_HEADER
                         + APT_CONFIG_GLOBAL_PROXY_HTTPS.format(
                             proxy_url="mock_https_proxy"
                         ),
@@ -996,7 +1084,7 @@ class TestAptProxyConfig:
                 [
                     mock.call(
                         APT_PROXY_CONF_FILE,
-                        messages.APT_PROXY_CONFIG_HEADER
+                        APT_PROXY_CONFIG_HEADER
                         + APT_CONFIG_GLOBAL_PROXY_HTTP.format(
                             proxy_url="mock_http_proxy"
                         )
@@ -1046,29 +1134,6 @@ class TestAptIsInstalled:
         assert expected == is_installed("test")
 
 
-class TestCompareVersion:
-    @pytest.mark.parametrize(
-        "ver1,ver2,relation,expected_result",
-        (
-            ("1.0", "2.0", "le", True),
-            ("1.0", "2.0", "gt", False),
-            ("2.0", "2.0", "lt", False),
-            ("2.0", "2.0", "eq", True),
-            ("2.0", "2.0", "gt", False),
-            ("2.1~18.04.1", "2.1", "le", True),
-            ("2.1", "2.1~18.04.1", "le", False),
-            ("2.10", "2.9", "ge", True),
-            ("2.10", "2.9", "lt", False),
-        ),
-    )
-    def test_compare_versions(
-        self, ver1, ver2, relation, expected_result, _subp
-    ):
-        """compare_versions returns True when the comparison is accurate."""
-        with mock.patch("uaclient.system._subp", side_effect=_subp):
-            assert expected_result is compare_versions(ver1, ver2, relation)
-
-
 class TestAptCache:
     @pytest.mark.parametrize(
         "file_exists,expected", ((True, 1.23), (False, None))
@@ -1097,7 +1162,8 @@ class TestAptCache:
     @mock.patch("uaclient.entitlements.esm.ESMInfraEntitlement")
     @mock.patch("uaclient.apt.system.is_current_series_lts")
     @mock.patch("uaclient.apt.system.is_current_series_active_esm")
-    @mock.patch("uaclient.apt.get_esm_cache")
+    @mock.patch("uaclient.apt.get_esm_apt_pkg_cache")
+    @mock.patch("uaclient.apt._ensure_esm_cache_structure")
     @mock.patch("uaclient.actions.status")
     @mock.patch("apt_pkg.config")
     @mock.patch("apt_pkg.init_config")
@@ -1106,6 +1172,7 @@ class TestAptCache:
         _m_apt_pkg_init_config,
         _m_apt_pkg_config,
         m_status,
+        _m_ensure_structure,
         m_esm_cache,
         m_is_esm,
         m_is_lts,
@@ -1224,10 +1291,7 @@ class TestGetAptConfigValues:
 
 
 class TestPreserveAptCfg:
-    def test_apt_config_is_preserved(
-        self,
-        apt_pkg,
-    ):
+    def test_apt_config_is_preserved(self):
         class AptDict(dict):
             def set(self, key, value):
                 super().__setitem__(key, value)
@@ -1237,16 +1301,15 @@ class TestPreserveAptCfg:
         apt_cfg["test1"] = [1, 2, 3]
         apt_cfg["test2"] = {"foo": "bar"}
 
-        type(apt_pkg).config = mock.PropertyMock(return_value=apt_cfg)
-
         def apt_func():
             apt_cfg["test"] = 3
             apt_cfg["test1"] = [3, 2, 1]
             apt_cfg["test2"] = {"foo": "test"}
             return apt_cfg
 
-        with PreserveAptCfg(apt_func):
-            pass
+        with mock.patch("apt_pkg.config", apt_cfg):
+            with PreserveAptCfg(apt_func):
+                pass
 
         assert 1 == apt_cfg["test"]
         assert [1, 2, 3] == apt_cfg["test1"]
@@ -1254,57 +1317,317 @@ class TestPreserveAptCfg:
 
 
 class TestGetPkgCandidateversion:
-    @pytest.mark.parametrize("check_esm_cache", ((True), (False)))
-    @mock.patch("uaclient.apt.get_apt_cache")
-    @mock.patch("uaclient.apt.get_esm_cache")
+    @pytest.mark.parametrize("check_esm_cache", (True, False))
+    @mock.patch("uaclient.apt.apt_pkg.DepCache")
+    @mock.patch("uaclient.apt.get_apt_pkg_cache")
+    @mock.patch("uaclient.apt.get_esm_apt_pkg_cache")
     def test_get_pkg_candidate_version(
         self,
         m_esm_cache,
         m_apt_cache,
+        m_dep_cache,
         check_esm_cache,
-        apt_pkg,
     ):
-        type(apt_pkg).config = mock.PropertyMock(return_value={})
-        m_pkg_ver = mock.MagicMock(version="1.2")
-        m_pkg = mock.MagicMock(candidate=m_pkg_ver)
+        m_pkg = mock.MagicMock()
         m_apt_cache.return_value = {"pkg1": m_pkg}
 
-        m_esm_pkg_ver = mock.MagicMock(version="1.3~esm1")
-        m_esm_pkg = mock.MagicMock(candidate=m_esm_pkg_ver)
+        m_esm_pkg = mock.MagicMock()
         m_esm_cache.return_value = {"pkg1": m_esm_pkg}
 
+        def depcache_candidate(package):
+            if package == m_pkg:
+                return mock.MagicMock(ver_str="1.2")
+            if package == m_esm_pkg:
+                return mock.MagicMock(ver_str="1.3~esm1")
+
+        m_dep_cache.return_value.get_candidate_ver = depcache_candidate
+
         actual_value = get_pkg_candidate_version("pkg1", check_esm_cache)
         if not check_esm_cache:
             assert "1.2" == actual_value
         else:
             assert "1.3~esm1" == actual_value
 
-    @mock.patch("uaclient.apt.get_apt_cache")
-    @mock.patch("uaclient.apt.get_esm_cache")
+    @mock.patch("uaclient.apt.apt_pkg.DepCache")
+    @mock.patch("uaclient.apt.get_apt_pkg_cache")
+    @mock.patch("uaclient.apt.get_esm_apt_pkg_cache")
     def test_get_pkg_candidate_version_when_esm_cache_fails(
         self,
         m_esm_cache,
         m_apt_cache,
-        apt_pkg,
+        m_dep_cache,
     ):
-        type(apt_pkg).config = mock.PropertyMock(return_value={})
-        m_pkg_ver = mock.MagicMock(version="1.2")
-        m_pkg = mock.MagicMock(candidate=m_pkg_ver)
-        m_apt_cache.return_value = {"pkg1": m_pkg}
+        def depcache_candidate(_package):
+            return mock.MagicMock(ver_str="1.2")
+
+        m_dep_cache.return_value.get_candidate_ver = depcache_candidate
+
+        m_apt_cache.return_value = {"pkg1": mock.MagicMock()}
         m_esm_cache.return_value = {}
 
         actual_value = get_pkg_candidate_version("pkg1", check_esm_cache=True)
         assert "1.2" == actual_value
 
-    @mock.patch("uaclient.apt.get_apt_cache")
-    def test_get_pkg_candidate_version_when_candidate_doesnt_exist(
+    @mock.patch("uaclient.apt.get_apt_pkg_cache")
+    def test_get_pkg_candidate_version_when_package_doesnt_exist(
         self,
         m_apt_cache,
-        apt_pkg,
     ):
-        type(apt_pkg).config = mock.PropertyMock(return_value={})
-        m_pkg = mock.MagicMock(candidate=None)
-        m_apt_cache.return_value = {"pkg1": m_pkg}
+        m_apt_cache.return_value = {}
 
         actual_value = get_pkg_candidate_version("pkg1")
         assert actual_value is None
+
+    @mock.patch("uaclient.apt.apt_pkg.DepCache")
+    @mock.patch("uaclient.apt.get_apt_pkg_cache")
+    def test_get_pkg_candidate_version_when_candidate_doesnt_exist(
+        self,
+        m_apt_cache,
+        m_dep_cache,
+    ):
+        def depcache_candidate(_package):
+            return None
+
+        m_dep_cache.return_value.get_candidate_ver = depcache_candidate
+        m_apt_cache.return_value = {"pkg1": mock.MagicMock()}
+
+        actual_value = get_pkg_candidate_version("pkg1")
+        assert actual_value is None
+
+
+class TestGetInstalledPackagesByOrigin:
+    @mock.patch("uaclient.apt.get_apt_pkg_cache")
+    def test_packages_by_origin(self, m_apt_cache):
+        origin_a_security = mock_origin(
+            "main", "series-security", "OriginA", ""
+        )
+        origin_a_updates = mock_origin("main", "series-updates", "OriginA", "")
+        origin_b = mock_origin("main", "series", "OriginB", "")
+        m_apt_cache.return_value.packages = [
+            mock_package("not_installed"),
+            mock_package("origin_a", mock_version("1.0", [origin_a_security])),
+            mock_package("origin_b", mock_version("1.0", [origin_b])),
+            mock_package(
+                "both_origins",
+                mock_version("1.0", [origin_a_updates, origin_b]),
+            ),
+            mock_package(
+                "origin_a_multiple_entries",
+                mock_version("1.0", [origin_a_security, origin_a_updates]),
+            ),
+            mock_package(
+                "origin_b_with_origin_a_not_installed",
+                mock_version("1.0", [origin_b]),
+                [mock_version("1.1", [origin_a_security, origin_a_updates])],
+            ),
+        ]
+
+        assert [
+            "both_origins",
+            "origin_a",
+            "origin_a_multiple_entries",
+        ] == sorted(
+            [p.name for p in get_installed_packages_by_origin("OriginA")]
+        )
+        assert [
+            "both_origins",
+            "origin_b",
+            "origin_b_with_origin_a_not_installed",
+        ] == sorted(
+            [p.name for p in get_installed_packages_by_origin("OriginB")]
+        )
+
+
+class TestGetRemoteVersionsForPackage:
+    def test_get_remote_versions_for_package(self):
+        assert ["0.9", "1.1"] == [
+            v.ver_str
+            for v in sorted(
+                get_remote_versions_for_package(
+                    mock_package(
+                        "name",
+                        installed_version=mock_version("1.0"),
+                        other_versions=[
+                            mock_version(
+                                "0.9", [mock_origin("", "", "Origin1", "")]
+                            ),
+                            mock_version(
+                                "1.1", [mock_origin("", "", "Origin2", "")]
+                            ),
+                        ],
+                    )
+                )
+            )
+        ]
+
+    def test_no_remote_versions(self):
+        assert [] == get_remote_versions_for_package(
+            mock_package("name", installed_version=mock_version("1.0"))
+        )
+
+    def test_get_remote_versions_excluding_origin(self):
+        assert ["0.9"] == [
+            v.ver_str
+            for v in sorted(
+                get_remote_versions_for_package(
+                    mock_package(
+                        "name",
+                        installed_version=mock_version(
+                            "1.0", [mock_origin("now", "", "Origin1", "")]
+                        ),
+                        other_versions=[
+                            mock_version(
+                                "0.9", [mock_origin("", "", "Origin1", "")]
+                            ),
+                            mock_version(
+                                "1.1", [mock_origin("", "", "Origin2", "")]
+                            ),
+                        ],
+                    ),
+                    exclude_origin="Origin2",
+                )
+            )
+        ]
+
+
+class TestUpdateSourcesList:
+    @mock.patch("uaclient.apt.os.path.join")
+    @mock.patch("apt_pkg.FileLock")
+    @mock.patch("uaclient.apt.AcquireProgress")
+    @mock.patch("apt_pkg.SourceList")
+    @mock.patch("apt_pkg.config")
+    @mock.patch("uaclient.apt.PreserveAptCfg")
+    def test_update_sources_list(
+        self,
+        m_preserve_apt_cfg,
+        m_apt_pkg_config,
+        m_apt_pkg_source_list,
+        m_acquire_progress,
+        _m_file_lock,
+        _m_join,
+    ):
+        update_sources_list("/sources.list")
+        assert (
+            mock.call("Dir::Etc::sourcelist", "/sources.list")
+            in m_apt_pkg_config.set.call_args_list
+        )
+        assert (
+            [
+                mock.call(
+                    m_acquire_progress.return_value,
+                    m_apt_pkg_source_list.return_value,
+                    0,
+                )
+            ]
+            == m_preserve_apt_cfg.return_value.__enter__.return_value.update.call_args_list  # noqa: E501
+        )
+
+
+@mock.patch("uaclient.apt.system.ensure_folder_absent")
+@mock.patch("uaclient.apt.system.create_file")
+@mock.patch("uaclient.apt.os.makedirs")
+class TestEsmCacheStructure:
+    def test_noop_when_present(
+        self, m_makedirs, m_create_file, m_folder_absent, tmpdir
+    ):
+        dir_path = tmpdir.strpath
+        # Both mocks because ESM_APT_ROOTDIR can't deal with it
+        with mock.patch("uaclient.apt.ESM_APT_ROOTDIR", dir_path), mock.patch(
+            "uaclient.apt.ESM_BASIC_FILE_STRUCTURE",
+            {
+                "files": [
+                    os.path.join(dir_path, "etc/apt/sources.list"),
+                    os.path.join(dir_path, "var/lib/dpkg/status"),
+                ],
+                "folders": [
+                    os.path.join(dir_path, "var/cache/apt/archives/partial"),
+                    os.path.join(dir_path, "var/lib/apt/lists/partial"),
+                ],
+            },
+        ):
+            # Guess what: cannot use tmp_path because the fixture is not on
+            # Xenial, and tmpdir does *not* support creating directories
+            # including parents (makedirs style)
+            tmpdir.mkdir("var")
+            tmpdir.mkdir("var/cache")
+            tmpdir.mkdir("var/cache/apt")
+            tmpdir.mkdir("var/cache/apt/archives")
+            tmpdir.mkdir("var/cache/apt/archives/partial")
+            tmpdir.mkdir("var/lib")
+            tmpdir.mkdir("var/lib/dpkg")
+            # yes
+            tmpdir.join("var/lib/dpkg/status").write("")
+            tmpdir.mkdir("var/lib/apt")
+            tmpdir.mkdir("var/lib/apt/lists")
+            tmpdir.mkdir("var/lib/apt/lists/partial")
+            tmpdir.mkdir("etc")
+            tmpdir.mkdir("etc/apt")
+            tmpdir.join("etc/apt/sources.list").write("")
+
+            _ensure_esm_cache_structure()
+
+        assert m_folder_absent.call_args_list == []
+        assert m_create_file.call_args_list == []
+        assert m_makedirs.call_args_list == []
+
+    @pytest.mark.parametrize("create_partial", (False, True))
+    def test_create_when_necessary(
+        self,
+        m_makedirs,
+        m_create_file,
+        m_folder_absent,
+        create_partial,
+        tmpdir,
+    ):
+        dir_path = tmpdir.strpath
+        with mock.patch("uaclient.apt.ESM_APT_ROOTDIR", dir_path), mock.patch(
+            "uaclient.apt.ESM_BASIC_FILE_STRUCTURE",
+            {
+                "files": [
+                    os.path.join(dir_path, "etc/apt/sources.list"),
+                    os.path.join(dir_path, "var/lib/dpkg/status"),
+                ],
+                "folders": [
+                    os.path.join(dir_path, "var/cache/apt/archives/partial"),
+                    os.path.join(dir_path, "var/lib/apt/lists/partial"),
+                ],
+            },
+        ):
+            if create_partial:
+                tmpdir.mkdir("etc")
+                tmpdir.mkdir("etc/apt")
+                tmpdir.join("etc/apt/sources.list").write("")
+
+            _ensure_esm_cache_structure()
+
+        assert m_folder_absent.call_args_list == [mock.call(tmpdir)]
+        assert m_create_file.call_args_list == [
+            mock.call(tmpdir + "/etc/apt/sources.list"),
+            mock.call(tmpdir + "/var/lib/dpkg/status"),
+        ]
+        assert m_makedirs.call_args_list == [
+            mock.call(
+                tmpdir + "/var/cache/apt/archives/partial",
+                exist_ok=True,
+                mode=755,
+            ),
+            mock.call(
+                tmpdir + "/var/lib/apt/lists/partial", exist_ok=True, mode=755
+            ),
+        ]
+
+
+class TestGetSystemSourcesFile:
+    @pytest.mark.parametrize(
+        "new_source_exists,expected_return",
+        (
+            (True, "/etc/apt/sources.list.d/ubuntu.sources"),
+            (False, "/etc/apt/sources.list"),
+        ),
+    )
+    @mock.patch("uaclient.apt.os.path.exists")
+    def test_get_system_sources_file(
+        self, m_exists, new_source_exists, expected_return
+    ):
+        m_exists.return_value = new_source_exists
+        assert get_system_sources_file() == expected_return
