@@ -11,8 +11,8 @@ from uaclient.contract import (
     API_V1_AVAILABLE_RESOURCES,
     API_V1_GET_CONTRACT_USING_TOKEN,
     UAContractClient,
-    _create_attach_forbidden_message,
     _get_override_weight,
+    _raise_attach_forbidden_message,
     _support_old_machine_info,
     apply_contract_overrides,
     get_available_resources,
@@ -343,7 +343,7 @@ class TestUAContractClient:
             "machine_id",
             "request_url_side_effect",
             "expected_machine_id",
-            "expected_create_attach_forbidden_message_call_args",
+            "expected_raise_attach_forbidden_message_call_args",
             "expected_raises",
             "expected_result",
         ],
@@ -409,7 +409,7 @@ class TestUAContractClient:
                 ],
                 mock.sentinel.get_machine_id,
                 [mock.call(mock.ANY)],
-                pytest.raises(exceptions.UserFacingError),
+                pytest.raises(exceptions.UbuntuProError),
                 None,
             ),
             (
@@ -430,7 +430,7 @@ class TestUAContractClient:
             ),
         ],
     )
-    @mock.patch("uaclient.contract._create_attach_forbidden_message")
+    @mock.patch("uaclient.contract._raise_attach_forbidden_message")
     @mock.patch("uaclient.contract._support_old_machine_info")
     @mock.patch("uaclient.contract.UAContractClient._get_activity_info")
     @mock.patch("uaclient.contract.UAContractClient.headers")
@@ -439,13 +439,13 @@ class TestUAContractClient:
         m_headers,
         m_get_activity_info,
         m_support_old_machine_info,
-        m_create_attach_forbidden_message,
+        m_raise_attach_forbidden_message,
         m_get_machine_id,
         m_request_url,
         machine_id,
         request_url_side_effect,
         expected_machine_id,
-        expected_create_attach_forbidden_message_call_args,
+        expected_raise_attach_forbidden_message_call_args,
         expected_raises,
         expected_result,
     ):
@@ -494,8 +494,8 @@ class TestUAContractClient:
             )
         ] == m_request_url.call_args_list
         assert (
-            expected_create_attach_forbidden_message_call_args
-            == m_create_attach_forbidden_message.call_args_list
+            expected_raise_attach_forbidden_message_call_args
+            == m_raise_attach_forbidden_message.call_args_list
         )
 
     def test_new_magic_attach_token_successfull(
@@ -528,9 +528,12 @@ class TestUAContractClient:
         "request_side_effect,expected_exception,message",
         (
             (
-                exceptions.UrlError("cause", "url"),
+                exceptions.ConnectivityError(cause="cause", url="url"),
                 exceptions.ConnectivityError,
-                messages.CONNECTIVITY_ERROR,
+                messages.E_CONNECTIVITY_ERROR.format(
+                    url="url",
+                    cause_error="cause",
+                ),
             ),
             (
                 [
@@ -543,7 +546,7 @@ class TestUAContractClient:
                     )
                 ],
                 exceptions.MagicAttachUnavailable,
-                messages.MAGIC_ATTACH_UNAVAILABLE,
+                messages.E_MAGIC_ATTACH_UNAVAILABLE,
             ),
         ),
     )
@@ -604,13 +607,21 @@ class TestUAContractClient:
         cfg = FakeConfig()
         client = UAContractClient(cfg)
         magic_token = "test-id"
-        request_url.side_effect = exceptions.UrlError("cause", "url")
+        request_url.side_effect = exceptions.ConnectivityError(
+            cause=Exception("cause"), url="url"
+        )
 
         with pytest.raises(exceptions.ConnectivityError) as exc_error:
             client.get_magic_attach_token_info(magic_token=magic_token)
 
-        assert messages.CONNECTIVITY_ERROR.msg == exc_error.value.msg
-        assert messages.CONNECTIVITY_ERROR.name == exc_error.value.msg_code
+        assert (
+            messages.E_CONNECTIVITY_ERROR.format(
+                url="url",
+                cause_error="cause",
+            ).msg
+            == exc_error.value.msg
+        )
+        assert messages.E_CONNECTIVITY_ERROR.name == exc_error.value.msg_code
 
     @pytest.mark.parametrize(
         "error_code,expected_exception",
@@ -651,13 +662,21 @@ class TestUAContractClient:
         cfg = FakeConfig()
         client = UAContractClient(cfg)
         magic_token = "test-id"
-        request_url.side_effect = exceptions.UrlError("cause", "url")
+        request_url.side_effect = exceptions.ConnectivityError(
+            cause=Exception("cause"), url="url"
+        )
 
         with pytest.raises(exceptions.ConnectivityError) as exc_error:
             client.revoke_magic_attach_token(magic_token=magic_token)
 
-        assert messages.CONNECTIVITY_ERROR.msg == exc_error.value.msg
-        assert messages.CONNECTIVITY_ERROR.name == exc_error.value.msg_code
+        assert (
+            messages.E_CONNECTIVITY_ERROR.format(
+                url="url",
+                cause_error="cause",
+            ).msg
+            == exc_error.value.msg
+        )
+        assert messages.E_CONNECTIVITY_ERROR.name == exc_error.value.msg_code
 
     @pytest.mark.parametrize(
         [
@@ -738,7 +757,13 @@ class TestUAContractClient:
                 "8001",
                 IsAttachedResult(is_attached=True),
                 mock.MagicMock(
-                    enabled_services=[helpers.mock_with_name_attr(name="one")]
+                    enabled_services=[
+                        helpers.mock_with_name_attr(
+                            name="one",
+                            variant_enabled=False,
+                            variant_name=None,
+                        )
+                    ]
                 ),
                 AttachmentData(
                     attached_at=datetime.datetime(
@@ -759,6 +784,7 @@ class TestUAContractClient:
                     "activityID": "activity_id",
                     "activityToken": "activity_token",
                     "resources": ["one"],
+                    "resourceVariants": {},
                     "lastAttachment": "2000-01-02T03:04:05+00:00",
                 },
             ),
@@ -786,7 +812,18 @@ class TestUAContractClient:
                 "8001",
                 IsAttachedResult(is_attached=True),
                 mock.MagicMock(
-                    enabled_services=[helpers.mock_with_name_attr(name="one")]
+                    enabled_services=[
+                        helpers.mock_with_name_attr(
+                            name="one",
+                            variant_enabled=False,
+                            variant_name=None,
+                        ),
+                        helpers.mock_with_name_attr(
+                            name="two",
+                            variant_enabled=True,
+                            variant_name="test",
+                        ),
+                    ]
                 ),
                 AttachmentData(
                     attached_at=datetime.datetime(
@@ -806,7 +843,8 @@ class TestUAContractClient:
                     "clientVersion": "8001",
                     "activityID": "machine_id",
                     "activityToken": "activity_token",
-                    "resources": ["one"],
+                    "resources": ["one", "two"],
+                    "resourceVariants": {"two": "test"},
                     "lastAttachment": "2000-01-02T03:04:05+00:00",
                 },
             ),
@@ -937,7 +975,7 @@ class TestProcessEntitlementDeltas:
             "Could not determine contract delta service type"
             " {{}} {}".format(new_access)
         )
-        with pytest.raises(exceptions.UserFacingError) as exc:
+        with pytest.raises(exceptions.UbuntuProError) as exc:
             process_entitlement_delta(
                 cfg=FakeConfig(), orig_access={}, new_access=new_access
             )
@@ -1015,13 +1053,15 @@ class TestCreateAttachForbiddenMessage:
         [
             "http_response",
             "expected_message",
+            "expected_info",
         ],
         [
             (
                 http.HTTPResponse(
                     code=403, headers={}, body="", json_list=[], json_dict={}
                 ),
-                messages.ATTACH_EXPIRED_TOKEN,
+                messages.E_ATTACH_EXPIRED_TOKEN,
+                {},
             ),
             (
                 http.HTTPResponse(
@@ -1049,17 +1089,16 @@ class TestCreateAttachForbiddenMessage:
                     },
                 ),
                 messages.NamedMessage(
-                    name=messages.ATTACH_FORBIDDEN_EXPIRED.name,
-                    msg=messages.ATTACH_FORBIDDEN.format(
-                        reason=messages.ATTACH_FORBIDDEN_EXPIRED.format(
-                            contract_id="contract-id", date="May 07, 2021"
-                        ).msg
+                    name=messages.E_ATTACH_FORBIDDEN_EXPIRED.name,
+                    msg=messages.E_ATTACH_FORBIDDEN_EXPIRED.format(
+                        contract_id="contract-id", date="May 07, 2021"
                     ).msg,
-                    additional_info={
-                        "contract_expiry_date": "05-07-2021",
-                        "contract_id": "contract-id",
-                    },
                 ),
+                {
+                    "contract_expiry_date": "05-07-2021",
+                    "contract_id": "contract-id",
+                    "date": "May 07, 2021",
+                },
             ),
             (
                 http.HTTPResponse(
@@ -1087,17 +1126,16 @@ class TestCreateAttachForbiddenMessage:
                     },
                 ),
                 messages.NamedMessage(
-                    name=messages.ATTACH_FORBIDDEN_NOT_YET.name,
-                    msg=messages.ATTACH_FORBIDDEN.format(
-                        reason=messages.ATTACH_FORBIDDEN_NOT_YET.format(
-                            contract_id="contract-id", date="May 07, 2021"
-                        ).msg
+                    name=messages.E_ATTACH_FORBIDDEN_NOT_YET.name,
+                    msg=messages.E_ATTACH_FORBIDDEN_NOT_YET.format(
+                        contract_id="contract-id", date="May 07, 2021"
                     ).msg,
-                    additional_info={
-                        "contract_effective_date": "05-07-2021",
-                        "contract_id": "contract-id",
-                    },
                 ),
+                {
+                    "contract_effective_date": "05-07-2021",
+                    "contract_id": "contract-id",
+                    "date": "May 07, 2021",
+                },
             ),
             (
                 http.HTTPResponse(
@@ -1116,22 +1154,22 @@ class TestCreateAttachForbiddenMessage:
                     },
                 ),
                 messages.NamedMessage(
-                    name=messages.ATTACH_FORBIDDEN_NEVER.name,
-                    msg=messages.ATTACH_FORBIDDEN.format(
-                        reason=messages.ATTACH_FORBIDDEN_NEVER.format(
-                            contract_id="contract-id"
-                        ).msg
+                    name=messages.E_ATTACH_FORBIDDEN_NEVER.name,
+                    msg=messages.E_ATTACH_FORBIDDEN_NEVER.format(
+                        contract_id="contract-id"
                     ).msg,
                 ),
+                {"contract_id": "contract-id"},
             ),
         ],
     )
-    def test_create_attach_forbidden_message(
-        self, http_response, expected_message
+    def test_raise_attach_forbidden_message(
+        self, http_response, expected_message, expected_info
     ):
-        assert expected_message == _create_attach_forbidden_message(
-            http_response
-        )
+        with pytest.raises(exceptions.UbuntuProError) as exc:
+            _raise_attach_forbidden_message(http_response)
+        assert exc.value.named_msg == expected_message
+        assert exc.value.additional_info == expected_info
 
 
 class TestGetAvailableResources:
@@ -1142,12 +1180,12 @@ class TestGetAvailableResources:
         """Raise error get_available_resources can't contact backend"""
         cfg = FakeConfig()
 
-        urlerror = exceptions.UrlError(
+        urlerror = exceptions.ConnectivityError(
             socket.gaierror(-2, "Name or service not known"), "url"
         )
         m_available_resources.side_effect = urlerror
 
-        with pytest.raises(exceptions.UrlError) as exc:
+        with pytest.raises(exceptions.ConnectivityError) as exc:
             get_available_resources(cfg)
         assert urlerror == exc.value
 
@@ -1594,8 +1632,8 @@ class TestRequestAutoAttach:
                 instance=mock.MagicMock()
             )
 
-        expected_message = messages.INVALID_PRO_IMAGE.format(
-            msg=error_response["message"]
+        expected_message = messages.E_INVALID_PRO_IMAGE.format(
+            error_msg=error_response["message"]
         )
         expected_args = [mock.call(error_response["message"])]
         assert expected_message.msg == exc_error.value.msg
