@@ -3,6 +3,7 @@ import pytest
 
 from lib.reboot_cmds import fix_pro_pkg_holds, main
 from uaclient import exceptions
+from uaclient.testing import fakes
 from uaclient.testing.helpers import does_not_raise
 
 M_FIPS_PATH = "uaclient.entitlements.fips.FIPSEntitlement."
@@ -42,10 +43,10 @@ class TestFixProPkgHolds:
             (
                 "enabled",
                 Exception(),
-                exceptions.UserFacingError(""),
+                fakes.FakeUbuntuProError(),
                 [mock.call()],
                 [mock.call(cleanup_on_failure=False)],
-                pytest.raises(exceptions.UserFacingError),
+                pytest.raises(exceptions.UbuntuProError),
             ),
         ],
     )
@@ -90,7 +91,7 @@ class TestFixProPkgHolds:
 )  # noqa: E501
 @mock.patch("lib.reboot_cmds.refresh_contract")
 @mock.patch("lib.reboot_cmds.fix_pro_pkg_holds")
-@mock.patch("uaclient.lock.SpinLock")
+@mock.patch("uaclient.lock.RetryLock")
 @mock.patch("lib.reboot_cmds._is_attached")
 @mock.patch(
     "uaclient.files.state_files.reboot_cmd_marker_file",
@@ -167,8 +168,13 @@ class TestMain:
         ],
         [
             (Exception(), 1),
-            (exceptions.LockHeldError("", "", 1), 1),
-            (exceptions.UserFacingError(""), 1),
+            (
+                exceptions.LockHeldError(
+                    lock_request="", lock_holder="", pid=1
+                ),
+                1,
+            ),
+            (fakes.FakeUbuntuProError(), 1),
         ],
     )
     def test_main_error_cases(
