@@ -10,10 +10,53 @@ from uaclient.snap import (
     get_config_option_value,
     get_installed_snaps,
     get_snap_info,
+    is_snapd_installed_as_a_snap,
     unconfigure_snap_proxy,
 )
 
 
+class TestIsSnapdInstalledAsASnap:
+    @pytest.mark.parametrize(
+        ["installed_snaps", "expected"],
+        [
+            ([], False),
+            (
+                [
+                    SnapPackage(
+                        "one", "123", "456", "oldest/unstable", "someone"
+                    )
+                ],
+                False,
+            ),
+            (
+                [
+                    SnapPackage(
+                        "snapd", "123", "456", "oldest/unstable", "someone"
+                    )
+                ],
+                True,
+            ),
+            (
+                [
+                    SnapPackage(
+                        "one", "123", "456", "oldest/unstable", "someone"
+                    ),
+                    SnapPackage(
+                        "snapd", "123", "456", "oldest/unstable", "someone"
+                    ),
+                ],
+                True,
+            ),
+        ],
+    )
+    @mock.patch("uaclient.snap.get_installed_snaps")
+    def test_is_snapd_installed_as_a_snap(
+        self, m_get_installed_snaps, installed_snaps, expected
+    ):
+        m_get_installed_snaps.return_value = installed_snaps
+        assert expected == is_snapd_installed_as_a_snap()
+
+
 class TestConfigureSnapProxy:
     @pytest.mark.parametrize("http_proxy", ("http_proxy", "", None))
     @pytest.mark.parametrize("https_proxy", ("https_proxy", "", None))
@@ -180,7 +223,7 @@ class TestGetSnapInfo:
         http_response_mock.read.return_value = b"invalid-json"
         m_http_connection.return_value = http_mock
 
-        with pytest.raises(exceptions.SnapdInvalidJson):
+        with pytest.raises(exceptions.InvalidJson):
             get_snap_info(snap="test")
 
         assert 1 == m_http_connection.call_count
