@@ -513,6 +513,7 @@ class TestStatus:
             }
             for cls in ENTITLEMENT_CLASSES
         ]
+        expected_services.sort(key=lambda x: x.get("name", ""))
         expected = copy.deepcopy(DEFAULT_STATUS)
         expected.update(
             {
@@ -852,13 +853,12 @@ class TestStatus:
                     "variants": variants,
                 }
             )
-        print("expected")
-        print(repr(expected))
+        expected["services"].sort(key=lambda x: x.get("name", ""))
         with mock.patch(
             "uaclient.status._get_config_status"
         ) as m_get_cfg_status:
             m_get_cfg_status.return_value = DEFAULT_CFG_STATUS
-            expected_status_calls = 12 if variants_in_contract else 9
+            expected_status_calls = 13 if variants_in_contract else 10
             assert expected == status.status(cfg=cfg, show_all=show_all)
             assert expected_status_calls == m_repo_uf_status.call_count
             assert 1 == m_livepatch_uf_status.call_count
