@@ -829,19 +829,21 @@ version,codename,series,created,release,eol,eol-server,eol-esm
         assert 7 == kinetic_di.eol_esm.month
         assert 2023 == kinetic_di.eol_esm.year
 
-        with pytest.raises(exceptions.UserFacingError) as excinfo:
+        with pytest.raises(exceptions.UbuntuProError) as excinfo:
             system.get_distro_info.__wrapped__("nonexistent")
         assert (
-            messages.MISSING_SERIES_IN_DISTRO_INFO_FILE.format("nonexistent")
-            == excinfo.value.msg
+            messages.E_MISSING_SERIES_IN_DISTRO_INFO_FILE.format(
+                series="nonexistent"
+            )
+            == excinfo.value.named_msg
         )
 
     def test_no_csv_file(self, m_load_file):
         m_load_file.side_effect = FileNotFoundError
 
-        with pytest.raises(exceptions.UserFacingError) as excinfo:
+        with pytest.raises(exceptions.UbuntuProError) as excinfo:
             system.get_distro_info.__wrapped__("focal")
-        assert messages.MISSING_DISTRO_INFO_FILE == excinfo.value.msg
+        assert messages.E_MISSING_DISTRO_INFO_FILE == excinfo.value.named_msg
 
 
 class TestGetReleaseInfo:
@@ -1214,9 +1216,12 @@ class TestSubp:
 
         logs = caplog_text()
         expected_logs = [
-            "'Funky apt %d error'. Retrying 3 more times.",
-            "'Funky apt %d error'. Retrying 2 more times.",
-            "'Funky apt %d error'. Retrying 1 more times.",
+            "Invalid command specified 'Funky apt %d error'.",
+            "Retrying 3 more times.",
+            "Invalid command specified 'Funky apt %d error'.",
+            "Retrying 2 more times.",
+            "Invalid command specified 'Funky apt %d error'.",
+            "Retrying 1 more times.",
         ]
         for log in expected_logs:
             assert log in logs
@@ -1331,6 +1336,31 @@ class TestIsSystemdServiceActive:
         assert expected_return == system.is_systemd_unit_active("test")
 
 
+class TestGetSystemdUnitActiveState:
+    @pytest.mark.parametrize(
+        [
+            "subp_side_effect",
+            "expected",
+        ],
+        (
+            ([("ActiveState=active\n", "")], "active"),
+            ([("ActiveState=activating\n", "")], "activating"),
+            ([("", "")], None),
+            ([("anything=anything", "")], None),
+            (exceptions.ProcessExecutionError("test"), None),
+        ),
+    )
+    @mock.patch("uaclient.system.subp")
+    def test_get_systemd_unit_active_state(
+        self,
+        m_subp,
+        subp_side_effect,
+        expected,
+    ):
+        m_subp.side_effect = subp_side_effect
+        assert expected == system.get_systemd_unit_active_state("test.service")
+
+
 class TestGetCpuInfo:
     @pytest.mark.parametrize(
         "cpuinfo,vendor_id,model,stepping",
@@ -1474,3 +1504,96 @@ class TestGetRebootRequiredPkgs:
         assert reboot_required_pkgs is not None
         assert expected_standard_pkgs == reboot_required_pkgs.standard_packages
         assert expected_kernel_pkgs == reboot_required_pkgs.kernel_packages
+
+
+@mock.patch("uaclient.system.util.we_are_currently_root")
+class TestGetInstalledUbuntuKernels:
+    def test_non_root_user_raises_error(self, m_is_root):
+        m_is_root.return_value = False
+
+        with pytest.raises(RuntimeError) as e:
+            system.get_installed_ubuntu_kernels()
+
+        assert "needs to be executed as root" in e.value.args[0]
+
+    @pytest.mark.parametrize(
+        "installed,files_in_boot,valid_file,expected_return",
+        (
+            (
+                ["linux-image-4.4.0-1020-kvm", "other_package"],
+                ["/boot/vmlinuz-4.4.0-1020-kvm"],
+                True,
+                ["4.4.0-1020-kvm"],
+            ),
+            (
+                ["linux-image-4.4.0-1020-kvm", "other_package"],
+                ["/boot/vmlinux-4.4.0-1020-kvm"],
+                True,
+                ["4.4.0-1020-kvm"],
+            ),
+            (
+                [
+                    "linux-image-5.4.0-39-generic",
+                    "linux-image-4.15.0-118-generic",
+                    "other package",
+                ],
+                [
+                    "/boot/vmlinuz-5.4.0-39-generic",
+                    "/boot/vmlinuz-4.15.0-118-generic",
+                ],
+                True,
+                ["5.4.0-39-generic", "4.15.0-118-generic"],
+            ),
+            (
+                ["some package", "other_package"],
+                ["/boot/vmlinuz-4.4.0-1020-kvm"],
+                True,
+                [],
+            ),
+            (
+                ["linux-image-4.4.0-1020-kvm", "other_package"],
+                [],
+                True,
+                [],
+            ),
+            (
+                ["linux-image-4.4.0-1020-kvm", "other_package"],
+                ["/boot/vmlinuz-4.4.0-1020-kvm"],
+                False,
+                [],
+            ),
+            (
+                ["linux-image-5.8.0-63-generic", "linux-image-5.6.0-1056-oem"],
+                [
+                    "/boot/vmlinuz-5.6.0-1056-oem",
+                    "/boot/vmlinuz-5.4.0-77-generic",
+                ],
+                True,
+                ["5.6.0-1056-oem"],
+            ),
+        ),
+    )
+    @mock.patch("uaclient.system.subp")
+    @mock.patch("uaclient.system.glob.glob")
+    @mock.patch("uaclient.apt.get_installed_packages_names")
+    def test_only_valid_packages_returned(
+        self,
+        m_package_names,
+        m_glob,
+        m_subp,
+        m_is_root,
+        installed,
+        files_in_boot,
+        valid_file,
+        expected_return,
+    ):
+        m_is_root.return_value = True
+        m_package_names.return_value = installed
+        m_glob.return_value = files_in_boot
+        m_subp.return_value = (
+            ("Some Linux kernel here", False)
+            if valid_file
+            else ("no good", False)
+        )
+
+        assert expected_return == system.get_installed_ubuntu_kernels()
