@@ -191,9 +191,10 @@ class TestUpdateMotdMessages:
                 [
                     mock.call(
                         mock.ANY,
-                        messages.CONTRACT_EXPIRES_SOON_MOTD.format(
+                        messages.CONTRACT_EXPIRES_SOON.pluralize(3).format(
                             remaining_days=3
-                        ),
+                        )
+                        + "\n\n",
                     )
                 ],
             ),
@@ -215,9 +216,10 @@ class TestUpdateMotdMessages:
                 [
                     mock.call(
                         mock.ANY,
-                        messages.CONTRACT_EXPIRED_GRACE_PERIOD_MOTD.format(
-                            remaining_days=11, expired_date="21 Dec 2012"
-                        ),
+                        messages.CONTRACT_EXPIRED_GRACE_PERIOD.pluralize(
+                            11
+                        ).format(remaining_days=11, expired_date="21 Dec 2012")
+                        + "\n\n",
                     )
                 ],
             ),
@@ -236,7 +238,7 @@ class TestUpdateMotdMessages:
                 True,
                 [mock.call(mock.ANY)],
                 [],
-                [mock.call(mock.ANY, messages.CONTRACT_EXPIRED_MOTD_NO_PKGS)],
+                [mock.call(mock.ANY, messages.CONTRACT_EXPIRED + "\n\n")],
             ),
             (
                 # expired, lts release, esm-apps not enabled
@@ -253,7 +255,7 @@ class TestUpdateMotdMessages:
                 True,
                 [mock.call(mock.ANY)],
                 [],
-                [mock.call(mock.ANY, messages.CONTRACT_EXPIRED_MOTD_NO_PKGS)],
+                [mock.call(mock.ANY, messages.CONTRACT_EXPIRED + "\n\n")],
             ),
             (
                 # expired, interim release
@@ -270,7 +272,7 @@ class TestUpdateMotdMessages:
                 True,
                 [mock.call(mock.ANY)],
                 [],
-                [mock.call(mock.ANY, messages.CONTRACT_EXPIRED_MOTD_NO_PKGS)],
+                [mock.call(mock.ANY, messages.CONTRACT_EXPIRED + "\n\n")],
             ),
             (
                 # expired, eol release, esm-infra enabled
@@ -290,9 +292,10 @@ class TestUpdateMotdMessages:
                 [
                     mock.call(
                         mock.ANY,
-                        messages.CONTRACT_EXPIRED_MOTD_PKGS.format(
-                            service="esm-infra", pkg_num=4
-                        ),
+                        messages.CONTRACT_EXPIRED_WITH_PKGS.pluralize(
+                            4
+                        ).format(service="esm-infra", pkg_num=4)
+                        + "\n\n",
                     )
                 ],
             ),
@@ -314,9 +317,10 @@ class TestUpdateMotdMessages:
                 [
                     mock.call(
                         mock.ANY,
-                        messages.CONTRACT_EXPIRED_MOTD_PKGS.format(
-                            service="esm-apps", pkg_num=5
-                        ),
+                        messages.CONTRACT_EXPIRED_WITH_PKGS.pluralize(
+                            5
+                        ).format(service="esm-apps", pkg_num=5)
+                        + "\n\n",
                     )
                 ],
             ),
